"""
Команды для управления стеками
"""

import click
from rich.console import Console

from core.api_client import ImagenariumAPIError
from core.services.stacks_service import StacksService
from utils.formatters import format_table, format_output, show_console_header

console = Console()


def _parse_string_value(value: str):
    """
    Пытается определить тип значения из строки

    Args:
        value: Строковое значение

    Returns:
        Значение с правильным типом (int, float, bool или str)
    """
    typed_value = value
    try:
        # Пробуем преобразовать в число
        if "." in value:
            typed_value = float(value)
        else:
            typed_value = int(value)
    except ValueError:
        # Если не число, пробуем bool
        if value.lower() in ("true", "false"):
            typed_value = value.lower() == "true"
    return typed_value


def _get_api_client(verbose: bool = False, ctx=None):
    """Создает API клиент с заданными параметрами"""
    from cli.main import get_api_client

    # Получаем параметры из контекста если доступен
    if ctx and hasattr(ctx, "obj"):
        server = ctx.obj.get("server")
        username = ctx.obj.get("username")
        password = ctx.obj.get("password")
        config = ctx.obj.get("config")
        return get_api_client(
            verbose=verbose,
            server=server,
            username=username,
            password=password,
            config=config,
        )
    else:
        return get_api_client(verbose=verbose)


@click.group()
def cli():
    """Управление стеками приложений"""
    pass


@cli.command("list")
@click.option(
    "--columns",
    help='Столбцы для отображения. Форматы: "NAME,STATUS" или "+COMPONENTS,-TIME". Доступные: NAME, NAMESPACE, VERSION, STATUS, REPO, COMMIT, TIME, TAG, PARAMS, COMPONENTS, PARAMS.<параметр>',
)
@click.option(
    "--filter",
    multiple=True,
    help="Фильтрация данных. Формат: COLUMN=value, COLUMN!=value, COLUMN~pattern. Примеры: STATUS=deployed, NAMESPACE!=test, NAME~postgres, PARAMS~database, PARAMS.DB_HOST=localhost",
)
@click.option("--limit", "-l", type=int, help="Ограничить количество записей")
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option(
    "--output",
    "-o",
    help='Формат вывода данных: json, yaml, tsv, или template (например: "{NAME} - {STATUS}")',
)
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def list_stacks(ctx, columns, filter, limit, no_cache, no_headers, output, verbose):
    """Показывает список стеков

    По умолчанию: NAME, NAMESPACE, VERSION, STATUS. Дополнительные: REPO, COMMIT, TIME, TAG, PARAMS, COMPONENTS, PARAMS.<параметр>
    """
    try:
        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли (скрываем при --no-headers или --output)
        show_console_header(api, show_header=not (no_headers or output))

        # Сервис стеков
        svc = StacksService(api_client=api)

        # Используем callback для прогресс-бара (вызывается только при реальных запросах)
        from core.progress import create_progress_context

        with create_progress_context(
                "Получение списка стеков...",
        ) as progress_callback:
            # columns_spec передается в list, где он парсится и применяется
            rows = svc.list(
                filters=list(filter) if filter else None,
                columns_spec=columns,
                no_cache=no_cache,
                progress=progress_callback,
            )
        if not rows:
            console.print("[yellow]Стеки не найдены[/yellow]")
            return

        if limit:
            rows = rows[:limit]

        # Определяем столбцы для вывода (данные уже отфильтрованы в list)
        from utils.formatters import parse_filters

        # Если columns указан, используем столбцы из данных, иначе default
        if columns and rows:
            columns_to_show = list(rows[0].keys()) if rows else svc.get_default_columns()
        else:
            columns_to_show = svc.get_default_columns()

        # Данные уже отфильтрованы по столбцам в list
        data = rows

        if not output or output == "table":
            highlight_filters = parse_filters(list(filter)) if filter else None
            # Подсветку имен стеков делаем в format_table
            format_table(
                data,
                columns=columns_to_show,
                no_headers=no_headers,
                highlight_filters=highlight_filters,
            )
        else:
            format_output(data, output, columns=columns_to_show)

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка получения списка стеков: {e}[/red]")
        raise click.Abort()


@cli.command("get")
@click.argument("stack_id")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def get_stack(ctx, stack_id, verbose):
    """Показывает детальную информацию о стеке"""
    try:
        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api)

        # Используем сервис для получения данных в унифицированном формате
        svc = StacksService(api_client=api)

        # Используем callback для прогресс-бара (вызывается только при реальных запросах)
        from core.progress import create_progress_context

        with create_progress_context(
                "Загрузка информации о стеке...",
        ) as progress_callback:
            stack = svc.get(stack_id, progress=progress_callback)

        if not stack:
            console.print(f"[red]Стек '{stack_id}' не найден[/red]")
            return

        # Выводим основную информацию (стек в snake_case формате)
        name = stack.get("name", "")
        if "@" in name:
            stack_name, namespace = name.split("@", 1)
            console.print(
                f"[bold white]NAME:[/bold white] {stack_name}[dim]@{namespace}[/dim]"
            )
        else:
            console.print(f"[bold white]NAME:[/bold white] {name}")
        namespace = stack.get("namespace", "")
        if namespace:
            console.print(f"[bold white]NAMESPACE:[/bold white] {namespace}")
        version = stack.get("version", "")
        if version:
            console.print(f"[bold white]VERSION:[/bold white] {version}")

        # Раскрашиваем статус
        from utils.highlighter import Highlighter

        status = stack.get("status", "UNKNOWN")
        styled_status = Highlighter.highlight(status, check_partial=True)
        console.print(f"[bold white]STATUS:[/bold white] {styled_status}")

        from utils.formatters import print_field

        print_field(stack, "repo")
        print_field(stack, "commit")
        # Преобразуем timestamp в нормальный вид
        timestamp = stack.get("time", "")
        if timestamp:
            try:
                from datetime import datetime
                dt = datetime.fromtimestamp(int(timestamp))
                time_str = dt.strftime("%Y-%m-%d %H:%M:%S")
                console.print(f"[bold white]TIME:[/bold white] {time_str}")
            except (ValueError, TypeError):
                console.print(f"[bold white]TIME:[/bold white] {timestamp}")
        tag = stack.get("tag", "")
        if tag:
            console.print(f"[bold white]TAG:[/bold white] {tag}")

        # Параметры: показываем динамические params.* столбцы, если они есть
        # Иначе используем поле params как fallback
        param_keys = [k for k in stack.keys() if k.startswith("params.") and not k.endswith("_raw")]
        if param_keys:
            console.print(f"[bold white]PARAMS:[/bold white]")
            for key in sorted(param_keys):
                value = stack.get(key, "")
                param_name = key.replace("params.", "")
                formatted_value = Highlighter.highlight_value(value)
                console.print(
                    f"[white]  [bold bright_white]{param_name}[/bold bright_white]: {formatted_value}[/white]")
        else:
            # Fallback: используем строковое поле params, если нет динамических полей
            params = stack.get("params", "")
            if params:
                console.print(f"[bold white]PARAMS:[/bold white]")
                for pair in params.split(", "):
                    if "=" in pair:
                        key, value = pair.split("=", 1)
                        # Попытка определить тип значения из строки
                        typed_value = _parse_string_value(value)
                        formatted_value = Highlighter.highlight_value(typed_value)
                        console.print(
                            f"[white]  [bold bright_white]{key}[/bold bright_white]: {formatted_value}[/white]")
            else:
                console.print(f"[bold white]PARAMS:[/bold white] [dim]Нет[/dim]")

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка получения информации о стеке: {e}[/red]")
        raise click.Abort()


# @cli.command("deploy")
# @click.argument("namespace")
# @click.argument("stack_id")
# @click.argument("version")
# @click.option("--repository", required=True, help="Git репозиторий")
# @click.option("--git-ref", required=True, help="Git ref (ветка или тег)")
# @click.option("--param", multiple=True, help="Параметр деплоя (формат: key=value)")
# @click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
# @click.pass_context
# def deploy_stack(
#     ctx, namespace, stack_id, version, repository, git_ref, param, verbose
# ):
#     """Деплоит стек в namespace"""
#     try:
#         # Создаем API клиент с нужными параметрами
#         api = _get_api_client(verbose=verbose, ctx=ctx)

#         # Собираем параметры деплоя
#         params = {}
#         for p in param:
#             if "=" in p:
#                 key, value = p.split("=", 1)
#                 params[key] = value
#             else:
#                 console.print(f"[yellow]Пропускаем неправильный параметр: {p}[/yellow]")

#         console.print(
#             f"[blue]Деплой стека {stack_id} версии {version} в namespace {namespace}...[/blue]"
#         )
#         api.deploy_stack(namespace, stack_id, version, repository, git_ref, params)
#         console.print(f"[green]Стек {stack_id} успешно задеплоен[/green]")

#     except ImagenariumAPIError as e:
#         console.print(f"[red]Ошибка деплоя стека: {e}[/red]")
#         raise click.Abort()


# @cli.command("undeploy")
# @click.argument("namespace")
# @click.argument("stack_id")
# @click.argument("version")
# @click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
# @click.pass_context
# def undeploy_stack(ctx, namespace, stack_id, version, verbose):
#     """Андеплоит стек из namespace"""
#     try:
#         # Создаем API клиент с нужными параметрами
#         api = _get_api_client(verbose=verbose, ctx=ctx)

#         console.print(
#             f"[blue]Андеплой стека {stack_id} версии {version} из namespace {namespace}...[/blue]"
#         )
#         result = api.undeploy_stack(namespace, stack_id, version)
#         console.print(f"[green]Стек {stack_id} успешно андеплоен[/green]")

#         if result:
#             console.print(f"Результат: {result}")

#     except ImagenariumAPIError as e:
#         console.print(f"[red]Ошибка андеплоя стека: {e}[/red]")
#         raise click.Abort()


def _parse_stack_spec(stack_spec: str, commit: str = None):
    """
    Парсит спецификацию стека в новом формате

    Поддерживаемые форматы:
    1) <stackId>@<namespace> - берем текущий repo, commit, stackId, namespace
    2) <stackId>@<namespace>:<тег> - берем текущий repo, stackId, namespace, commit из тега
    3) <stackId>@<namespace> <commit> - берем текущий repo, stackId, namespace, commit из параметров

    Args:
        stack_spec: Спецификация стека
        commit: Дополнительный параметр commit (для формата 3)

    Returns:
        tuple: (stack_id, namespace, version, git_ref, repo_name)
    """
    # Формат 2: <stackId>@<namespace>:<тег>
    if ":" in stack_spec and "@" in stack_spec:
        # Находим последнее двоеточие (тег может содержать двоеточия)
        colon_pos = stack_spec.rfind(":")
        stack_part = stack_spec[:colon_pos]
        tag_part = stack_spec[colon_pos + 1:]

        if "@" in stack_part:
            stack_id, namespace = stack_part.split("@", 1)
            # Извлекаем commit из тега (например, dev__2025_08_05_10_17-6ca3d9f -> 6ca3d9f)
            if "__" in tag_part and "-" in tag_part:
                # Формат: dev__2025_08_05_10_17-6ca3d9f
                commit_from_tag = tag_part.split("-")[-1]
            else:
                # Если формат неожиданный, используем весь тег как commit
                commit_from_tag = tag_part

            return stack_id, namespace, None, commit_from_tag, None

    # Формат 1 и 3: <stackId>@<namespace> [commit]
    if "@" in stack_spec:
        stack_id, namespace = stack_spec.split("@", 1)
        return stack_id, namespace, None, commit, None

    # Fallback: старая логика
    return stack_spec, None, None, commit, None


def _get_stack_info_from_components(
        api,
        stack_id: str,
        namespace: str,
        no_cache: bool = False,
        progress=None,
):
    """
    Получает информацию о стеке из компонентов

    Args:
        api: API клиент
        stack_id: ID стека
        namespace: Namespace
        no_cache: Не использовать кэш
        progress: Callback для отображения прогресса

    Returns:
        dict: Информация о стеке или None
    """
    try:
        components = api.get_components(no_cache=no_cache, progress=progress)

        for group in components:
            group_namespace = group.get("namespace", "")
            if group_namespace != namespace:
                continue

            stacks = group.get("stacks", [])
            for stack in stacks:
                if stack.get("stackId") == stack_id:
                    return stack

        return None
    except Exception:
        return None


@cli.command("template")
@click.argument("stack_spec", metavar="STACK_SPEC")
@click.argument("commit", required=False, metavar="[COMMIT]")
@click.option(
    "--diff", is_flag=True, help="Показать diff между текущим commit и указанным COMMIT"
)
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def get_template(ctx, stack_spec, commit, diff, no_cache, no_headers, verbose):
    """Получает и отображает шаблон деплоя для указанного стека.

    Поддерживает три формата спецификации стека:

    \b
    ФОРМАТ 1: STACK_ID@NAMESPACE
        Получает шаблон для текущего состояния стека.
        Автоматически извлекает репозиторий и commit из развернутого стека.

    \b
    ФОРМАТ 2: STACK_ID@NAMESPACE:TAG
        Получает шаблон для указанного тега.
        Commit извлекается из тега (например, dev__2025_08_05_10_17-6ca3d9f → 6ca3d9f).

    \b
    ФОРМАТ 3: STACK_ID@NAMESPACE COMMIT
        Получает шаблон для указанного commit.
        Позволяет явно указать конкретный commit для получения шаблона.

    \b
    РЕЖИМ DIFF:
        При указании --diff сравнивает текущий шаблон с шаблоном для указанного COMMIT.
        Показывает "Шаблоны не различаются" если они одинаковые, иначе выводит оба шаблона.
        Требует обязательного указания COMMIT.

    \b
    ПАРАМЕТРЫ:
        STACK_SPEC    Спецификация стека в одном из поддерживаемых форматов
        COMMIT        Git commit hash (только для формата 3)
    """
    try:
        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        # Парсим спецификацию стека
        stack_id, namespace, version, git_ref, repo_name = _parse_stack_spec(
            stack_spec, commit
        )

        if not stack_id or not namespace:
            console.print(
                f"[red]Неверный формат спецификации стека: {stack_spec}[/red]"
            )
            console.print("Поддерживаемые форматы:")
            console.print("  <stackId>@<namespace>")
            console.print("  <stackId>@<namespace>:<тег>")
            console.print("  <stackId>@<namespace> <commit>")
            raise click.Abort()

        # Используем callback для прогресс-бара
        from core.progress import create_progress_context

        with create_progress_context(
                "Получение информации о шаблоне стека...",
        ) as progress_callback:
            # Получаем информацию о стеке из компонентов
            stack_info = _get_stack_info_from_components(
                api, stack_id, namespace, no_cache, progress=progress_callback
            )

        if not stack_info:
            console.print(f"[red]Стек {stack_id}@{namespace} не найден[/red]")
            raise click.Abort()

        # Извлекаем необходимые параметры
        repo_name = stack_info.get("repo")
        if not repo_name:
            console.print(
                f"[red]Не удалось получить репозиторий для стека {stack_id}@{namespace}[/red]"
            )
            raise click.Abort()

        version = stack_info.get("version", "latest")

        # Если git_ref не указан, берем текущий commit из стека
        if not git_ref:
            git_ref = stack_info.get("commit")
            if not git_ref:
                console.print(
                    f"[red]Не удалось получить commit для стека {stack_id}@{namespace}[/red]"
                )
                raise click.Abort()

        # Проверяем, нужен ли режим diff
        if diff:
            # Режим diff - сравниваем шаблоны
            if not commit:
                console.print(f"[red]Для режима --diff необходимо указать COMMIT[/red]")
                raise click.Abort()

            # Используем callback для прогресс-бара
            from core.progress import create_progress_context

            with create_progress_context(
                    "Получение diff шаблонов...",
            ) as progress_callback:
                # Получаем diff
                diff_result = api.diff_template(
                    namespace, stack_id, version, commit, progress=progress_callback
                )

            # Выводим основную информацию в стиле других команд get (только если не --no-headers)
            if not no_headers:
                console.print(
                    f"[bold white]NAME:[/bold white] {stack_id}[dim]@{namespace}[/dim]"
                )
                console.print(f"[bold white]NAMESPACE:[/bold white] {namespace}")
                console.print(f"[bold white]VERSION:[/bold white] {version}")
                console.print(f"[bold white]REPO:[/bold white] {repo_name}")
                console.print(f"[bold white]CURRENT_COMMIT:[/bold white] {git_ref}")
                console.print(f"[bold white]DIFF_COMMIT:[/bold white] {commit}")
                console.print()

            # Проверяем, различаются ли шаблоны
            if diff_result.get("equals", False):
                console.print("[green]Шаблоны не различаются[/green]")
            else:
                console.print("[yellow]Шаблоны различаются:[/yellow]")
                console.print()
                console.print("[bold]Текущий шаблон:[/bold]")
                console.print(diff_result.get("oldTemplate", ""))
                console.print()
                console.print("[bold]Новый шаблон:[/bold]")
                console.print(diff_result.get("newTemplate", ""))
        else:
            # Обычный режим - получаем шаблон
            # Используем callback для прогресс-бара
            from core.progress import create_progress_context

            with create_progress_context(
                    "Получение шаблона деплоя...",
            ) as progress_callback:
                # Получаем шаблон
                template = api.get_template(
                    repo_name, stack_id, version, git_ref, progress=progress_callback
                )

            # Выводим основную информацию в стиле других команд get (только если не --no-headers)
            if not no_headers:
                console.print(
                    f"[bold white]NAME:[/bold white] {stack_id}[dim]@{namespace}[/dim]"
                )
                console.print(f"[bold white]NAMESPACE:[/bold white] {namespace}")
                console.print(f"[bold white]VERSION:[/bold white] {version}")
                console.print(f"[bold white]REPO:[/bold white] {repo_name}")
                console.print(f"[bold white]COMMIT:[/bold white] {git_ref}")
                console.print()

            # Выводим шаблон
            console.print(template)

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка получения шаблона: {e}[/red]")
        raise click.Abort()



"""
Главный модуль CLI для imgctl
"""

import os

import click
from rich.console import Console

from commands import nodes, stacks, components, registries, repositories, servers, logs, shell
from core.api_client import ImagenariumAPIClient

console = Console()


def show_version():
    """Показать информацию о версии приложения"""
    VERSION = "3.0.14"
    print(
        f"imgctl v{VERSION} - Консольная утилита для управления контейнерной платформой «Imagenarium»"
    )
    print("Документация: https://gitlab.com/koden8/imgctl | Автор: @koden8")


def custom_help_callback(ctx, param, value):
    """Кастомный callback для help с локализацией"""
    if value and not ctx.resilient_parsing:
        show_extended_help()
        ctx.exit()


def show_extended_help():
    """Показывает расширенный help с командами и подкомандами в простом текстовом формате"""
    print(
        "imgctl v3.0.14 - Консольная утилита для управления контейнерной платформой «Imagenarium»"
    )
    print("Документация: https://gitlab.com/koden8/imgctl | Автор: @koden8")
    print()

    print("Глобальные параметры:")
    print("  --server, -s        Имя сервера или URL для подключения")
    print("  --username, -u      Имя пользователя (для прямого подключения)")
    print("  --password, -p      Пароль (для прямого подключения)")
    print("  --config, -c        Путь к файлу конфигурации")
    print("  --verbose, -v       Подробный вывод HTTP запросов")
    print("  --version           Показать версию")
    print("  --help, -h          Показать справку")
    print()

    print("Основные команды:")
    print("  components       Управление компонентами приложений")
    print("  stacks           Управление стеками приложений")
    print("  logs             Просмотр логов компонентов")
    print("  nodes            Управление нодами кластера")
    print("  registries       Управление реестрами образов")
    print("  repositories     Управление репозиториями")
    print("  servers          Управление серверами и подключениями")
    print("  shell            Интерактивная оболочка (REPL)")
    print()

    print("Команды components:")
    print("  list             Показывает список развернутых компонентов")
    print("  get              Показывает детальную информацию о компоненте")
    print("  start            Запускает компонент")
    print("  stop             Останавливает компонент")
    print("  tags             Показывает теги развертываний для компонента")
    print("  upgrade          Обновляет компоненты до указанных тегов")
    print()

    print("Команды stacks:")
    print("  list             Показывает список стеков")
    print("  get              Показывает детальную информацию о стеке")
    print("  deploy           Деплоит стек в namespace")
    print("  undeploy         Андеплоит стек из namespace")
    print(
        "  template         Получает шаблон деплоя (поддерживает 3 формата и режим --diff)"
    )
    print()

    print("Команды servers:")
    print("  list             Показывает список серверов")
    print("  add              Добавляет новый сервер")
    print("  remove           Удаляет сервер")
    print("  set-default      Устанавливает сервер по умолчанию")
    print()

    print("Справка:")
    print("  imgctl <команда> --help     Показать справку по команде")
    print("  imgctl --help               Показать общую справку")
    print("  imgctl --version            Показать версию")


def get_api_client(
        verbose: bool = False,
        server: str = None,
        username: str = None,
        password: str = None,
        config: str = None,
):
    """Создает и возвращает API клиент с заданными параметрами"""
    # Инициализация конфигурации
    cfg = None

    try:
        # Используем централизованный метод для определения конфигурации
        from core.services.servers_service import ServersService

        try:
            cfg = ServersService.get_config(
                server=server,
                username=username,
                password=password,
                config_path=config,
            )
        except ValueError as e:
            console.print(f"[red]{e}[/red]")
            raise click.Abort()

        # Определяем имя сервера для настроек TTL и кеша
        server_name_for_ttl = ServersService.get_current_server(server=server)

        # Если не удалось определить из параметров/окружения, пытаемся извлечь из конфигурации
        if not server_name_for_ttl and cfg and cfg.server:
            from core.services.servers_service import get_server_name_from_config_or_url
            server_name_for_ttl = get_server_name_from_config_or_url(server_url=cfg.server)

        # async_mode только в shell режиме
        is_shell_mode = os.getenv("IMGCTL_SHELL_MODE") == "1"

        # Инициализация API клиента с контекстом (подключение, async, кеширование)
        return ImagenariumAPIClient(
            config=cfg,
            verbose=verbose,
            async_mode=is_shell_mode,
            server_name=server_name_for_ttl
        )

    except Exception as e:
        console.print(f"[red]Ошибка инициализации: {e}[/red]")
        console.print(
            "[yellow]Попробуйте добавить сервер командой 'servers add'[/yellow]"
        )
        raise click.Abort()


@click.group(
    context_settings={"help_option_names": ["-h", "--help"], "show_default": False},
    invoke_without_command=True,
)
@click.option("--server", "-s", help="Имя сервера или URL для подключения")
@click.option("--username", "-u", help="Имя пользователя (для прямого подключения)")
@click.option("--password", "-p", help="Пароль (для прямого подключения)")
@click.option("--config", "-c", help="Путь к файлу конфигурации")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод")
@click.option(
    "--version",
    is_flag=True,
    help="Показать информацию о версии приложения",
    callback=lambda ctx, param, value: show_version() or ctx.exit() if value else None,
)
@click.option(
    "--help",
    "-h",
    is_flag=True,
    help="Показать это сообщение и выйти",
    callback=custom_help_callback,
)
@click.pass_context
def main(ctx, server, username, password, config, verbose, version, help):
    """imgctl - Консольная утилита для управления контейнерной платформой «Imagenarium»"""
    ctx.ensure_object(dict)

    ctx.obj["verbose"] = verbose
    ctx.obj["server"] = server
    ctx.obj["username"] = username
    ctx.obj["password"] = password
    ctx.obj["config"] = config

    # Если это главная команда без аргументов, показываем расширенный help
    if ctx.invoked_subcommand is None and not ctx.params.get("help"):
        show_extended_help()
        ctx.exit()

    # Если это команда servers или shell, не инициализируем API клиент
    if ctx.invoked_subcommand in ["servers", "shell"]:
        # Но для shell сохраняем все параметры для последующего использования
        # (они уже сохранены в ctx.obj выше)
        return

    # Если это команда с подкомандами без подкоманды, показываем help
    if ctx.invoked_subcommand in [
        "components",
        "stacks",
        "nodes",
        "registries",
        "repositories",
    ] and not ctx.params.get("help"):
        # Проверяем, есть ли подкоманда в args
        import sys

        if len(sys.argv) <= 2 or sys.argv[2] in ["-h", "--help"]:
            return

    # Если это команда с подкомандами, не создаем API клиент здесь
    if ctx.invoked_subcommand in [
        "components",
        "stacks",
        "nodes",
        "registries",
        "repositories",
    ]:
        return

    # Инициализация конфигурации
    cfg = None

    try:
        # Используем централизованный метод для определения конфигурации
        from core.services.servers_service import ServersService

        try:
            cfg = ServersService.get_config(
                server=server,
                username=username,
                password=password,
                config_path=config,
            )
        except ValueError as e:
            console.print(f"[red]{e}[/red]")
            raise click.Abort()

        ctx.obj["config"] = cfg

        # Определяем имя сервера для настроек TTL и кеша

        server_name_for_ttl = ServersService.get_current_server(server=server)

        # Если не удалось определить из параметров/окружения, пытаемся извлечь из конфигурации
        if not server_name_for_ttl and cfg and cfg.server:
            from core.services.servers_service import get_server_name_from_config_or_url
            server_name_for_ttl = get_server_name_from_config_or_url(server_url=cfg.server)

        # async_mode только в shell режиме
        is_shell_mode = os.getenv("IMGCTL_SHELL_MODE") == "1"

        # Инициализация API клиента с контекстом (подключение, async, кеширование)
        ctx.obj["api"] = ImagenariumAPIClient(
            config=cfg,
            verbose=verbose,
            async_mode=is_shell_mode,
            server_name=server_name_for_ttl
        )

    except Exception as e:
        console.print(f"[red]Ошибка инициализации: {e}[/red]")
        console.print(
            "[yellow]Попробуйте добавить сервер командой 'servers add'[/yellow]"
        )
        raise click.Abort()


# Добавляем команды
main.add_command(servers.cli, name="servers")
main.add_command(nodes.cli, name="nodes")
main.add_command(stacks.cli, name="stacks")
main.add_command(components.cli, name="components")
main.add_command(registries.cli, name="registries")
main.add_command(repositories.cli, name="repositories")
main.add_command(logs.logs, name="logs")
main.add_command(shell.shell, name="shell")

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Команды для работы с логами компонентов
"""

import html
import re
import time
import webbrowser

import click
from rich.console import Console
from rich.text import Text

from core.api_client import ImagenariumAPIError
from utils.formatters import show_console_header

console = Console()


def _get_api_client(verbose: bool = False, ctx=None):
    """Создает API клиент с заданными параметрами"""
    from cli.main import get_api_client

    # Получаем параметры из контекста если доступен
    if ctx and hasattr(ctx, "obj"):
        server = ctx.obj.get("server")
        username = ctx.obj.get("username")
        password = ctx.obj.get("password")
        config = ctx.obj.get("config")
        return get_api_client(
            verbose=verbose,
            server=server,
            username=username,
            password=password,
            config=config,
        )
    else:
        return get_api_client(verbose=verbose)


def parse_log_line(line):
    """Парсит строку лога из SSE потока"""
    if not line.startswith("data:"):
        return None

    # Убираем префикс 'data:'
    html_content = line[5:]

    # Проверяем на служебные сообщения (ping, heartbeat и т.д.)
    if html_content.strip() in ["$ping$", "$heartbeat$", "ping", "pong", "keepalive"]:
        return {
            "timestamp": "",
            "level": "PING",
            "level_color": "",
            "message": html_content.strip(),
            "raw": line,
            "is_service": True,
        }

    # Извлекаем timestamp
    timestamp_match = re.search(
        r"<span style='color:#ffeca5'>(.*?)</span>", html_content
    )
    timestamp = timestamp_match.group(1) if timestamp_match else ""

    # Извлекаем уровень лога
    level_match = re.search(
        r"<span style='color:([^']+)'>\[(.*?)\]</span>", html_content
    )
    level = level_match.group(2) if level_match else ""
    level_color = level_match.group(1) if level_match else ""

    # Убираем все HTML теги и декодируем HTML-сущности
    clean_content = re.sub(r"<[^>]+>", "", html_content)
    clean_content = clean_content.replace("&nbsp;", " ")
    clean_content = html.unescape(clean_content)

    # Убираем timestamp и level из начала строки
    if timestamp:
        clean_content = clean_content.replace(timestamp, "", 1)
    if level:
        clean_content = clean_content.replace(f"[{level}]", "", 1)

    # Очищаем от лишних пробелов, но сохраняем переносы строк для многострочных сообщений
    clean_content = clean_content.strip()

    return {
        "timestamp": timestamp,
        "level": level,
        "level_color": level_color,
        "message": clean_content,
        "raw": line,
        "is_service": False,
    }


def format_log_line(log_data, show_timestamp=True):
    """Форматирует строку лога для вывода в консоль"""
    if not log_data:
        return ""

    # Пропускаем служебные сообщения (ping, heartbeat)
    if log_data.get("is_service", False):
        return None

    timestamp = log_data["timestamp"]
    level = log_data["level"]
    message = log_data["message"]
    is_service = log_data.get("is_service", False)

    # Создаем Rich Text объект для цветного вывода
    text = Text()

    if show_timestamp and timestamp:
        text.append(f"[{timestamp}] ", style="bright_yellow")

    # Цветовое кодирование уровня лога
    if level:
        if level.upper() == "ERROR":
            text.append(f"[{level}] ", style="bold red")
        elif level.upper() == "WARN":
            text.append(f"[{level}] ", style="bold yellow")
        elif level.upper() == "INFO":
            text.append(f"[{level}] ", style="bold blue")
        elif level.upper() == "DEBUG":
            text.append(f"[{level}] ", style="dim white")
        elif level.upper() == "PING":
            text.append(f"[{level}] ", style="dim cyan")
        else:
            text.append(f"[{level}] ", style="white")

    # Специальное форматирование для служебных сообщений
    if is_service:
        text.append(message, style="dim cyan")
    else:
        text.append(message, style="white")

    return text


def get_component_info(api, component_name, no_cache=False, verbose=True):
    """Получает информацию о компоненте, используя кэш"""
    if no_cache or not api.cache:
        return _find_component_in_list(api, component_name)

    # Используем новый метод поиска ID с кешированием на 1 час
    component_id = api.find_component_id_by_name(component_name)

    if not component_id:
        if verbose:
            console.print(f"[dim]Компонент {component_name} не найден[/dim]")
        return None

    # Если ID найден, получаем полную информацию о компоненте
    if verbose:
        console.print(
            f"[dim]Компонент {component_name} найден (ID: {component_id})[/dim]"
        )

    # Создаем минимальную информацию о компоненте с ID
    # Не вызываем _find_component_in_list, чтобы не сбрасывать кеш
    component_info = {"id": component_id, "name": component_name}

    return component_info


def _find_component_in_list(api, component_name):
    """Ищет компонент в списке развертываний"""
    try:
        components = api.get_components()

        for group in components:
            stacks = group.get("stacks", [])
            for stack in stacks:
                stack_components = stack.get("components", [])
                for comp in stack_components:
                    if comp.get("name") == component_name:
                        return comp
        return None
    except Exception as e:
        console.print(f"[dim]Ошибка поиска компонента: {e}[/dim]")
        return None


def stream_logs(api, component_id, lines=None, follow=False, level_filter=None):
    """Потоковое чтение логов компонента"""
    try:
        # Параметры запроса
        params = {"tzOffset": 0}

        # В режиме follow запрашиваем только последние записи
        if follow:
            params["follow"] = "true"
            # Если не указано количество строк, берем только последние 10
            if lines is None:
                params["lines"] = 10
                console.print(
                    "[dim]Режим follow: показываем последние 10 записей + новые[/dim]"
                )
            else:
                params["lines"] = lines
        elif lines:
            params["lines"] = lines

        # Заголовки для SSE
        headers = {
            "Accept": "text/event-stream",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Accept-Encoding": "gzip, deflate",
        }

        response = api.session.get(
            f"{api.base_url}/logs/{component_id}",
            params=params,
            headers=headers,
            stream=True,
            timeout=None if follow else 30,
        )

        if response.status_code == 404:
            # Пробрасываем ошибку 404 для обработки в вызывающем коде
            raise Exception(
                f"There is no container, service or task with ID = '{component_id}'"
            )
        elif response.status_code != 200:
            console.print(f"[red]Ошибка получения логов: {response.status_code}[/red]")
            return

        line_count = 0
        ping_count = 0
        processed_count = 0  # Счетчик всех обработанных строк
        start_time = time.time()

        # Буфер для накопления логов перед выводом (для правильного порядка)
        log_buffer = []
        # Переменная для отслеживания последнего timestamp и level
        last_timestamp = None
        last_level = None

        try:
            for line in response.iter_lines(decode_unicode=True):
                processed_count += 1
                if line and line.startswith("data:"):
                    # Парсим строку лога
                    log_data = parse_log_line(line)
                    if not log_data:
                        continue

                    # Считаем служебные сообщения отдельно
                    if log_data.get("is_service", False):
                        ping_count += 1
                        # Скрываем служебные сообщения
                        continue

                    # Фильтрация по уровню (только для обычных логов)
                    if (
                            level_filter
                            and not log_data.get("is_service", False)
                            and log_data["level"].upper() != level_filter.upper()
                    ):
                        continue

                    # Обработка многострочных сообщений
                    current_timestamp = log_data.get("timestamp", "")
                    current_level = log_data.get("level", "")
                    current_message = log_data.get("message", "")

                    # Если у текущей строки нет timestamp и level, но есть предыдущие,
                    # то это продолжение многострочного сообщения
                    if (
                            not current_timestamp
                            and not current_level
                            and last_timestamp
                            and last_level
                    ):
                        # Это продолжение предыдущего сообщения
                        # Проверяем, является ли это первой строкой ASCII art
                        if current_message and current_message.strip().startswith("."):
                            # Первая строка ASCII art - без timestamp, но с отступом
                            log_data["timestamp"] = ""
                            log_data["level"] = ""
                            # Добавляем отступ в начало сообщения для выравнивания
                            # Нужно добавить пробел в начало, чтобы выровнять с остальными строками
                            log_data["message"] = " " + current_message.strip()
                        else:
                            # Остальные строки многострочного сообщения - с timestamp
                            log_data["timestamp"] = last_timestamp
                            log_data["level"] = last_level
                    else:
                        # Обновляем последние значения
                        last_timestamp = current_timestamp
                        last_level = current_level

                    # Форматируем строку лога
                    formatted_line = format_log_line(log_data, show_timestamp=True)
                    if formatted_line is not None:
                        # В режиме follow выводим сразу для реального времени
                        if follow:
                            console.print(formatted_line)
                        else:
                            # В обычном режиме накапливаем в буфере для правильного порядка
                            log_buffer.append(formatted_line)

                        line_count += 1

                    # Ограничение количества строк (только для обычных логов)
                    if lines and line_count >= lines:
                        if not follow:
                            # В обычном режиме выходим после показа нужного количества строк
                            break
                        else:
                            # В режиме follow показываем последние N записей, затем продолжаем следить
                            if line_count == lines:
                                # Сбрасываем счетчики для новых записей (без вывода сообщения)
                                line_count = 0
                                ping_count = 0

                    # Небольшая задержка для читаемости (только в обычном режиме)
                    if not follow:
                        time.sleep(0.01)

                # Защита от зависания: если обработано много строк, но не найдено логов с нужным уровнем
                if (
                        not follow
                        and level_filter
                        and processed_count > 1000
                        and line_count == 0
                ):
                    console.print(
                        f"[yellow]Не найдено логов с уровнем {level_filter} среди {processed_count} обработанных записей[/yellow]"
                    )
                    break

                # Дополнительная защита: если запрашивается определенное количество строк, но их нет
                if (
                        not follow
                        and lines
                        and processed_count > lines * 10
                        and line_count == 0
                ):
                    console.print(
                        f"[yellow]Не найдено логов среди {processed_count} обработанных записей[/yellow]"
                    )
                    break

        except KeyboardInterrupt:
            pass

        finally:
            # В обычном режиме выводим накопленные логи из буфера
            if not follow and log_buffer:
                # Сортируем логи по timestamp для правильного порядка
                def extract_timestamp(log_item):
                    # Извлекаем timestamp из Rich Text объекта
                    try:
                        text_str = str(log_item)
                        # Ищем timestamp в формате [timestamp] (первое вхождение)
                        import re

                        timestamp_match = re.search(r"\[([^\]]+)\]", text_str)
                        if timestamp_match:
                            timestamp = timestamp_match.group(1)
                            # Преобразуем timestamp в формат для сортировки
                            # Формат: 05.10.25 20:31:19.819
                            try:
                                # Убираем точки из даты и двоеточия из времени для сортировки
                                clean_timestamp = (
                                    timestamp.replace(".", "")
                                    .replace(":", "")
                                    .replace(" ", "")
                                )
                                return clean_timestamp
                            except:
                                return timestamp
                    except:
                        pass
                    return "00000000000000000"  # Возвращаем минимальное значение для сортировки

                # Сортируем логи по timestamp
                try:
                    log_buffer.sort(key=extract_timestamp)
                except Exception:
                    # Если сортировка не удалась, выводим как есть
                    pass

                for formatted_line in log_buffer:
                    console.print(formatted_line)

            response.close()

    except Exception as e:
        # Пробрасываем исключение для обработки в вызывающем коде
        raise


@click.command()
@click.argument("component_name")
@click.option("--browser", "-b", is_flag=True, help="Открыть логи в браузере")
@click.option(
    "--follow", "-f", is_flag=True, help="Следить за логами в реальном времени"
)
@click.option(
    "--lines",
    "-n",
    type=int,
    help="Количество строк логов (по умолчанию: 50, или все при --follow)",
)
@click.option(
    "--tail",
    "-t",
    type=int,
    help="Количество последних записей для показа (по умолчанию: 10 в режиме follow, 50 в обычном режиме)",
)
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def logs(
        ctx,
        component_name,
        browser,
        follow,
        lines,
        tail,
        no_cache,
        no_headers,
        verbose,
):
    """Просмотр логов компонентов"""
    try:
        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        # Используем callback для прогресс-бара
        from core.progress import create_progress_context

        with create_progress_context(
                f"Поиск компонента {component_name}...",
        ) as progress_callback:
            # Используем существующий метод с callback
            component_id = api.find_component_id_by_name(
                component_name,
                no_cache=no_cache,
                progress=progress_callback,
            )
            if component_id:
                component_info = {"id": component_id, "name": component_name}
            else:
                component_info = None

        if not component_info:
            console.print(f"[red]Компонент {component_name} не найден в кэше[/red]")
            console.print("[yellow]Доступные компоненты:[/yellow]")

            # Показываем доступные компоненты из кэша или загружаем заново
            try:
                components = api.get_components()
                for group in components:
                    namespace = group.get("namespace", "Unknown")
                    stacks = group.get("stacks", [])
                    for stack in stacks:
                        stack_components = stack.get("components", [])
                        for comp in stack_components:
                            comp_name = comp.get("name", "Unknown")
                            console.print(f"  - {comp_name} (namespace: {namespace})")
            except Exception as e:
                console.print(
                    f"[dim]Не удалось загрузить список компонентов: {e}[/dim]"
                )
            return

        # Получаем ID компонента
        component_id = component_info.get("id")
        if not component_id:
            console.print(
                f"[red]Не удалось получить ID компонента {component_name}[/red]"
            )
            return

        console.print(
            f"[green]Найден компонент: {component_name} (ID: {component_id})[/green]"
        )

        # Если нужно открыть в браузере
        if browser:
            logview_url = f"{api.base_url}/logview/{component_id}"
            console.print(f"[blue]Открываем логи в браузере: {logview_url}[/blue]")
            webbrowser.open(logview_url)
            return

        # Настройка параметров
        if tail is not None:
            # Если указан --tail, используем его значение
            lines = tail
        elif lines is None:
            if follow:
                lines = 50  # В режиме follow показываем последние 50 записей
            else:
                lines = 50  # Обычный режим - показываем 50 записей

        # Если не указан --follow, работаем в обычном режиме (не follow)
        if not follow:
            follow = False

        # Потоковое чтение логов с обработкой ошибок кеша
        try:
            stream_logs(
                api, component_id, lines=lines, follow=follow, level_filter=None
            )
        except Exception as e:
            error_msg = str(e)
            # Перехватываем любые ошибки при получении логов (включая timeout)
            if (
                    "There is no container, service or task with ID" in error_msg
                    or "Read timed out" in error_msg
                    or "Connection" in error_msg
            ):
                console.print(
                    f"[yellow]Task ID {component_id} не найден, обновляем кеш...[/yellow]"
                )

                # Инвалидируем кеш для этого компонента
                cache_key = f"component_info:{component_name}"
                if api.cache:
                    api.cache.invalidate(cache_key)
                    console.print(f"[dim]Кеш для {component_name} инвалидирован[/dim]")

                # Получаем свежую информацию о компоненте
                console.print(
                    f"[blue]Повторный поиск компонента {component_name}...[/blue]"
                )
                component_info = get_component_info(
                    api, component_name, no_cache=True
                )

                if not component_info:
                    console.print(
                        f"[red]Компонент {component_name} не найден после обновления кеша[/red]"
                    )
                    return

                # Получаем новый ID компонента
                new_component_id = component_info.get("id")

                if not new_component_id:
                    console.print(
                        f"[red]Не удалось получить новый ID компонента {component_name}[/red]"
                    )
                    return

                console.print(
                    f"[green]Найден компонент: {component_name} (ID: {new_component_id})[/green]"
                )

                # Повторяем попытку с новым ID
                stream_logs(
                    api,
                    new_component_id,
                    lines=lines,
                    follow=follow,
                    level_filter=level,
                )
            else:
                # Если это другая ошибка, просто пробрасываем её
                raise

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка API: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Ошибка: {e}[/red]")
        raise click.Abort()



"""
Утилиты для работы с динамическими столбцами (ENV, PARAMS)
"""

from typing import List, Dict, Any


def normalize_column_name(column_name: str, all_columns: List[str]) -> str:
    """
    Нормализует имя столбца: принимает UPPER_CASE от пользователя, возвращает snake_case для использования в коде

    Args:
        column_name: Имя столбца от пользователя (может быть в UPPER_CASE или snake_case)
        all_columns: Список всех доступных столбцов (в snake_case или с сохранением регистра для динамических)

    Returns:
        Нормализованное имя столбца для использования в коде
    """
    # Преобразуем входное имя в lowercase для поиска
    column_lower = column_name.lower()

    # Для динамических столбцов (env.*, params.*) проверяем разные варианты регистра
    if column_lower.startswith("env.") or column_lower.startswith("params."):
        # Сначала проверяем точное совпадение (в исходном регистре)
        if column_name in all_columns:
            return column_name
        # Затем проверяем lowercase версию
        if column_name.lower() in all_columns:
            return column_name.lower()
        # Если не найдено, ищем колонку с нечувствительным к регистру сравнением
        # (динамические колонки могут быть с любым регистром ключа)
        prefix = column_lower.split(".", 1)[0] + "."  # "params." или "env."
        suffix = column_name.split(".", 1)[1] if "." in column_name else ""  # "CLIENT_PORT"
        suffix_lower = suffix.lower()  # "client_port"

        # Ищем колонку с тем же префиксом и суффиксом (нечувствительно к регистру)
        for col in all_columns:
            col_lower = col.lower()
            if col_lower.startswith(prefix):
                col_suffix = col.split(".", 1)[1] if "." in col else ""
                # Сравниваем суффиксы нечувствительно к регистру
                if col_suffix.lower() == suffix_lower:
                    return col

        # Если ничего не найдено, возвращаем lowercase версию
        return column_name.lower()
    else:
        # Для остальных столбцов: ищем в snake_case (все столбцы теперь в snake_case)
        if column_lower in all_columns:
            return column_lower
        # Если не найден, возвращаем lowercase версию
        return column_lower


def parse_columns_with_env_support(
        columns_spec: str, default_columns: List[str], all_columns: List[str]
) -> List[str]:
    """
    Парсит спецификацию столбцов с поддержкой ENV столбцов

    Args:
        columns_spec: Спецификация столбцов (например: "NAME,STATUS" или "+REPLICAS,-NAMESPACE")
        default_columns: Столбцы по умолчанию
        all_columns: Все доступные столбцы

    Returns:
        Список столбцов для отображения
    """
    if not columns_spec:
        return default_columns

    # Разделяем по запятым и убираем пробелы
    parts = [part.strip() for part in columns_spec.split(",")]

    # Проверяем, есть ли префиксы +/-
    has_prefixes = any(part.startswith(("+", "-")) for part in parts)

    if has_prefixes:
        # Режим с префиксами: начинаем с столбцов по умолчанию
        result_columns = default_columns.copy()

        for part in parts:
            if part.startswith("+"):
                # Добавляем столбец
                column_spec = part[1:]
                column = normalize_column_name(column_spec, all_columns)
                if column in all_columns and column not in result_columns:
                    result_columns.append(column)
            elif part.startswith("-"):
                # Удаляем столбец
                column_spec = part[1:]
                column = normalize_column_name(column_spec, all_columns)
                if column in result_columns:
                    result_columns.remove(column)
            else:
                # Обычный столбец (без префикса) - заменяем весь список
                result = []
                for p in parts:
                    column = normalize_column_name(p, all_columns)
                    if column in all_columns:
                        result.append(column)
                return result

        return result_columns
    else:
        # Обычный режим: явное указание столбцов
        result = []
        for part in parts:
            column = normalize_column_name(part, all_columns)
            if column in all_columns:
                result.append(column)

        return result


def parse_filters_with_env_support(filters: List[str]) -> List[Dict[str, Any]]:
    """
    Парсит список фильтров с поддержкой ENV столбцов

    Args:
        filters: Список строк фильтров

    Returns:
        Список словарей с информацией о фильтрах
    """
    parsed_filters = []

    for filter_str in filters:
        # Поддерживаемые операторы
        operators = ["!=", ">=", "<=", "~", "=", ">", "<"]

        # Находим оператор в строке
        operator = None
        for op in operators:
            if op in filter_str:
                operator = op
                break

        if not operator:
            continue

        # Разделяем на колонку и значение
        parts = filter_str.split(operator, 1)
        if len(parts) != 2:
            continue

        column_spec = parts[0].strip()
        # Преобразуем имя столбца (принимаем UPPER_CASE от пользователя)
        # Для динамических колонок (env.*, params.*) нормализуем без списка all_columns
        # так как они могут быть в любом регистре в данных
        if column_spec.lower().startswith(('env.', 'params.')):
            # Для env.* и params.* сохраняем структуру, но нормализуем регистр префикса
            # Суффикс (имя ключа) может быть в любом регистре в данных
            parts_col = column_spec.split('.', 1)
            if len(parts_col) == 2:
                prefix = parts_col[0].lower()  # "env" или "params"
                suffix = parts_col[1]  # "CLIENT_PORT" - сохраняем исходный регистр
                column = f"{prefix}.{suffix}"
            else:
                column = column_spec.lower()
        else:
            # Для остальных колонок используем lowercase
            column = column_spec.lower()
        value = parts[1].strip()

        # Убираем кавычки если есть
        if value.startswith('"') and value.endswith('"'):
            value = value[1:-1]
        elif value.startswith("'") and value.endswith("'"):
            value = value[1:-1]

        parsed_filters.append({"column": column, "operator": operator, "value": value})

    return parsed_filters


def convert_params_filters_to_raw(
        filters: List[Dict[str, Any]],
) -> List[Dict[str, Any]]:
    """
    Преобразует фильтры PARAMS в фильтры по исходным значениям (до маскирования)

    Args:
        filters: Список фильтров

    Returns:
        Список фильтров с преобразованными именами столбцов
    """
    converted_filters = []

    for filter_item in filters:
        column = filter_item["column"]

        # Если это фильтр по params.<параметр>, заменяем на params.<параметр>_raw
        if column.startswith("params.") and not column.endswith("_raw"):
            # Проверяем, есть ли соответствующий _raw столбец
            raw_column = column + "_raw"
            converted_filters.append(
                {
                    "column": raw_column,
                    "operator": filter_item["operator"],
                    "value": filter_item["value"],
                }
            )
        else:
            # Оставляем фильтр как есть
            converted_filters.append(filter_item)

    return converted_filters


def convert_env_filters_to_raw(filters: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    """
    Преобразует фильтры ENV в фильтры по исходным значениям (до маскирования)

    Args:
        filters: Список фильтров

    Returns:
        Список фильтров с преобразованными именами столбцов
    """
    converted_filters = []

    for filter_item in filters:
        column = filter_item["column"]

        # Если это фильтр по env.<переменная>, заменяем на env.<переменная>_raw
        if column.startswith("env.") and not column.endswith("_raw"):
            # Проверяем, есть ли соответствующий _raw столбец
            raw_column = column + "_raw"
            converted_filters.append(
                {
                    "column": raw_column,
                    "operator": filter_item["operator"],
                    "value": filter_item["value"],
                }
            )
        else:
            # Оставляем фильтр как есть
            converted_filters.append(filter_item)

    return converted_filters



"""
Команды для управления компонентами
"""

from typing import Optional

import click
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn

from core.api_client import ImagenariumAPIError
from core.services.components_service import ComponentsService
from utils.formatters import format_table, format_output, show_console_header
from utils.verbose import verbose_print, set_verbose

console = Console()


def _parse_string_value(value: str):
    """
    Пытается определить тип значения из строки

    Args:
        value: Строковое значение

    Returns:
        Значение с правильным типом (int, float, bool или str)
    """
    typed_value = value
    try:
        # Пробуем преобразовать в число
        if "." in value:
            typed_value = float(value)
        else:
            typed_value = int(value)
    except ValueError:
        # Если не число, пробуем bool
        if value.lower() in ("true", "false"):
            typed_value = value.lower() == "true"
    return typed_value


def _get_api_client(verbose: bool = False, ctx=None):
    """Создает API клиент с заданными параметрами"""
    from cli.main import get_api_client

    # Получаем параметры из контекста если доступен
    if ctx and hasattr(ctx, "obj"):
        server = ctx.obj.get("server")
        username = ctx.obj.get("username")
        password = ctx.obj.get("password")
        config = ctx.obj.get("config")
        return get_api_client(
            verbose=verbose,
            server=server,
            username=username,
            password=password,
            config=config,
        )
    else:
        return get_api_client(verbose=verbose)


@click.group()
def cli():
    """Управление компонентами приложений"""
    pass


@cli.command("list")
@click.option("--limit", "-l", type=int, help="Ограничить количество записей")
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option(
    "--columns",
    help='Столбцы для отображения. Форматы: "NAME,STATUS" или "+REPLICAS,-NAMESPACE". Доступные: NAME, NAMESPACE, STACK, IMAGE, TAG, STACK_VERSION, STATUS, REPLICAS, PORTS, UPDATED, CREATED, REPO, COMMIT, ADMIN_MODE, RUN_APP, SHOW_ADMIN_MODE, OFF, NETWORKS, ENV, ENV.<переменная>',
)
@click.option(
    "--filter",
    multiple=True,
    help="Фильтрация данных. Формат: COLUMN=value, COLUMN!=value, COLUMN~pattern. Примеры: STATUS=RUNNING, NAMESPACE!=test, NAME~postgres, ENV~database, ENV.DB_HOST=localhost",
)
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option(
    "--output",
    "-o",
    help='Формат вывода данных: json, yaml, tsv, или template (например: "{NAME} - {STATUS}")',
)
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def list_components(ctx, limit, no_cache, columns, filter, no_headers, output, verbose):
    """Показывает список развернутых компонентов

    По умолчанию: NAME, NAMESPACE, IMAGE, TAG, UPDATED, STATUS. Дополнительные: STACK, STACK_VERSION, REPLICAS, PORTS, CREATED, REPO, COMMIT, ADMIN_MODE, RUN_APP, SHOW_ADMIN_MODE, OFF, NETWORKS, ENV, ENV.<переменная>
    """
    try:
        # Устанавливаем режим verbose для логирования
        set_verbose(verbose)

        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли (скрываем при --no-headers или --output)
        show_console_header(api, show_header=not (no_headers or output))

        # Сервис данных
        svc = ComponentsService(api_client=api)

        # Используем callback для прогресс-бара (вызывается только при реальных запросах)
        from core.progress import create_progress_context

        with create_progress_context(
                "Получение списка компонентов...",
        ) as progress_callback:
            # Получаем данные и применяем фильтры на уровне сервиса
            # columns_spec передается в list, где он парсится и применяется
            rows = svc.list(
                filters=list(filter) if filter else None,
                columns_spec=columns,
                no_cache=no_cache,
                progress=progress_callback,
            )

        if not rows:
            console.print("Компоненты не найдены")
            return

        # Ограничение
        if limit:
            rows = rows[:limit]

        # Определяем столбцы для вывода (данные уже отфильтрованы в list)
        from utils.formatters import parse_filters

        # Если columns указан, используем столбцы из данных, иначе default
        if columns and rows:
            requested_columns = list(rows[0].keys()) if rows else svc.get_default_columns()
        else:
            requested_columns = svc.get_default_columns()

        # Данные уже отфильтрованы по столбцам в list
        filtered_data = rows

        # Вывод
        if not output or output == "table":
            highlight_filters = parse_filters(list(filter)) if filter else None
            # Используем truncate из сервиса для image
            # Подсветку имен компонентов делаем ПОСЛЕ подсветки фильтров в format_table
            for row in filtered_data:
                if "image" in row:
                    row["image"] = svc.truncate_image_name(row["image"])
            format_table(
                filtered_data,
                columns=requested_columns,
                no_headers=no_headers,
                highlight_filters=highlight_filters,
            )
        else:
            format_output(filtered_data, output, columns=requested_columns)

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка получения списка компонентов: {e}[/red]")
        raise click.Abort()


@cli.command("stop")
@click.argument("name")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.pass_context
def stop_component(ctx, name, no_headers):
    """Останавливает компонент"""
    try:
        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=False, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        console.print(f"[blue]Остановка компонента {name}...[/blue]")
        svc = ComponentsService(api_client=api)
        svc.stop(name)
        console.print(f"[green]Компонент {name} успешно остановлен[/green]")

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка остановки компонента: {e}[/red]")
        raise click.Abort()


@cli.command("start")
@click.argument("name")
@click.option("--logs", is_flag=True, help="Показать логи компонента после запуска")
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.pass_context
def start_component(ctx, name, logs, no_cache, no_headers):
    """Запускает компонент"""
    try:
        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=False, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        # Запускаем компонент (как в stop - без лишних проверок)
        console.print(f"[blue]Запуск компонента {name}...[/blue]")
        svc = ComponentsService(api_client=api)
        svc.start(name)
        console.print(f"[green]Компонент {name} успешно запущен[/green]")

        # Если запрошены логи, показываем их
        if logs:
            import time
            from commands.logs import stream_logs, get_component_info

            console.print(f"[blue]Показ логов компонента {name}...[/blue]")

            # Небольшая задержка, чтобы компонент успел запуститься
            time.sleep(2)

            try:
                # Получаем информацию о компоненте для получения ID
                # Сначала пробуем использовать кеш
                component_info = get_component_info(api, name, no_cache=False)
                if component_info:
                    # Получаем ID компонента
                    component_id = component_info.get("id")

                    if component_id:
                        try:
                            stream_logs(
                                api,
                                component_id,
                                lines=5,
                                follow=False,
                                level_filter=None,
                            )
                        except Exception as e:
                            error_msg = str(e)
                            # Перехватываем любые ошибки при получении логов (включая timeout)
                            if (
                                    "There is no container, service or task with ID"
                                    in error_msg
                                    or "Read timed out" in error_msg
                                    or "Connection" in error_msg
                            ):
                                console.print(
                                    f"[yellow]Task ID {component_id} не найден, обновляем кеш...[/yellow]"
                                )

                                # Инвалидируем кеш для этого компонента
                                cache_key = f"component_info:{name}"
                                if api.cache:
                                    api.cache.invalidate(cache_key)
                                    console.print(
                                        f"[dim]Кеш для {name} инвалидирован[/dim]"
                                    )

                                # Получаем свежую информацию о компоненте
                                console.print(
                                    f"[blue]Повторный поиск компонента {name}...[/blue]"
                                )
                                component_info = get_component_info(
                                    api, name, no_cache=True
                                )

                                if not component_info:
                                    console.print(
                                        f"[red]Компонент {name} не найден после обновления кеша[/red]"
                                    )
                                    return

                                # Получаем новый ID компонента
                                new_component_id = component_info.get("id")

                                if not new_component_id:
                                    console.print(
                                        f"[red]Не удалось получить новый ID компонента {name}[/red]"
                                    )
                                    return

                                console.print(
                                    f"[green]Найден компонент: {name} (ID: {new_component_id})[/green]"
                                )

                                # Повторяем попытку с новым ID
                                stream_logs(
                                    api,
                                    new_component_id,
                                    lines=5,
                                    follow=False,
                                    level_filter=None,
                                )
                            else:
                                # Если это другая ошибка, просто пробрасываем её
                                raise
                    else:
                        console.print(
                            f"[red]Не удалось получить ID компонента {name}[/red]"
                        )
                else:
                    console.print(
                        f"[red]Компонент {name} не найден для логирования[/red]"
                    )
            except Exception as e:
                console.print(f"[red]Ошибка показа логов: {e}[/red]")

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка запуска компонента: {e}[/red]")
        raise click.Abort()


@cli.command("get")
@click.argument("name")
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def get_component(ctx, name, no_cache, verbose):
    """Показывает детальную информацию о компоненте"""
    try:
        # Устанавливаем режим verbose для логирования
        set_verbose(verbose)

        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api)

        # Используем сервис для получения данных в унифицированном формате
        svc = ComponentsService(api_client=api)

        # Используем callback для прогресс-бара (вызывается только при реальных запросах)
        from core.progress import create_progress_context

        with create_progress_context(
                "Получение информации о компонентах...",
        ) as progress_callback:
            component = svc.get(name, no_cache=no_cache, progress=progress_callback)

        if not component:
            console.print(f"Компонент '{name}' не найден")
            return

        # Выводим основную информацию (компонент уже в унифицированном формате с ключами NAME, NAMESPACE и т.д.)

        # Основные параметры (данные в snake_case)
        from utils.highlighter import Highlighter

        component_name = component.get("name", "")
        component_namespace = component.get("namespace", "")
        highlighted_name = Highlighter.highlight_component_name(
            component_name, component_namespace
        )
        console.print(f"[bold white]NAME:[/bold white] {highlighted_name}")
        console.print(f"[bold white]NAMESPACE:[/bold white] {component_namespace}")

        # Подсвечиваем имя стека
        stack = component.get("stack", "")
        if stack:
            if "@" in stack:
                stack_name, namespace = stack.split("@", 1)
                console.print(
                    f"[bold white]STACK:[/bold white] {stack_name}[dim]@{namespace}[/dim]"
                )
            else:
                console.print(f"[bold white]STACK:[/bold white] {stack}")

        stack_version = component.get("stack_version", "")
        if stack_version:
            console.print(f"[bold white]STACK_VERSION:[/bold white] {stack_version}")

        # Получаем тег и образ (данные в snake_case)
        image = component.get("image", "")
        tag = component.get("tag", "")

        console.print(f"[bold white]IMAGE:[/bold white] {image}")
        highlighted_tag = Highlighter.highlight_tag(tag)
        console.print(f"[bold white]TAG:[/bold white] {highlighted_tag}")

        # Статус (данные в snake_case)
        status = component.get("status", "")
        styled_status = Highlighter.highlight(status, check_partial=True)
        console.print(f"[bold white]STATUS:[/bold white] {styled_status}")

        # Используем утилиту для вывода полей
        from utils.formatters import print_field

        # Реплики, время создания и обновления
        print_field(component, "replicas")
        print_field(component, "created")
        print_field(component, "updated")

        # Информация о репозитории
        print_field(component, "repo")
        print_field(component, "commit")

        # Дополнительные параметры (булевы значения показываются только если True)
        print_field(component, "admin_mode")
        print_field(component, "run_app")
        print_field(component, "show_admin_mode")
        print_field(component, "off")

        # Порты
        print_field(component, "ports")

        # Переменные окружения
        # Сначала пробуем динамические env.* поля (если есть)
        env_keys = [k for k in component.keys() if k.startswith("env.") and not k.endswith("_raw")]
        if env_keys:
            console.print(f"[bold white]ENV:[/bold white]")
            for key in sorted(env_keys):
                value = component.get(key, "")
                env_name = key.replace("env.", "")
                highlighted_value = Highlighter.highlight_value(value)
                console.print(f"[white]  [bold bright_white]{env_name}[/bold bright_white]: {highlighted_value}[/white]")
        else:
            # Fallback: используем строковое поле env
            env = component.get("env", "")
            if env:
                console.print(f"[bold white]ENV:[/bold white]")
                # ENV уже отформатирован как строка "KEY1=value1, KEY2=value2"
                env_pairs = [pair.strip() for pair in env.split(",")]
                for env_pair in env_pairs:
                    if "=" in env_pair:
                        key, value = env_pair.split("=", 1)
                        highlighted_value = Highlighter.highlight_value(value)
                        console.print(
                            f"[white]  [bold bright_white]{key}[/bold bright_white]: {highlighted_value}[/white]")

        # Тома
        volumes = component.get("_volumes", [])
        if volumes:
            console.print(f"[bold white]VOLUMES:[/bold white]")
            for volume in volumes:
                source = volume.get("source", "")
                target = volume.get("target", "")
                driver = volume.get("driver", "")
                if driver:
                    console.print(f"[white]  {source} -> {target} ({driver})[/white]")
                else:
                    console.print(f"[white]  {source} -> {target}[/white]")

        # Сетевые подключения
        networks = component.get("_networks", [])
        if networks:
            console.print(f"[bold white]NETWORKS:[/bold white]")
            for network in networks:
                name = network.get("name", "")
                driver = network.get("driver", "")
                subnet = network.get("subnet", "")
                if subnet:
                    console.print(f"  {name} ({driver}) - {subnet}")
                elif driver:
                    console.print(f"  {name} ({driver})")
                else:
                    console.print(f"  {name}")

        # Привязки
        binds = component.get("_binds", [])
        if binds:
            console.print(f"[bold white]BINDS:[/bold white]")
            for bind in binds:
                source = bind.get("source", "")
                target = bind.get("target", "")
                console.print(f"  {source} -> {target}")

        # Метки
        labels = component.get("_labels", [])
        if labels:
            console.print(f"[bold white]LABELS:[/bold white]")
            # labels может быть списком словарей или одним словарем
            if isinstance(labels, list):
                for label in labels:
                    for key, value in label.items():
                        console.print(f"[white]  {key}={value}[/white]")
            elif isinstance(labels, dict):
                for key, value in labels.items():
                    console.print(f"[white]  {key}={value}[/white]")

    except ImagenariumAPIError as e:
        console.print(f"Ошибка получения информации о компоненте: {e}")
        raise click.Abort()


@cli.command("tags")
@click.argument("name")
@click.option("--limit", "-l", type=int, help="Ограничить количество результатов")
@click.option("--no-cache", is_flag=True, help="Не использовать кеш")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.pass_context
def component_tags(
        ctx,
        name: str,
        limit: Optional[int],
        no_cache: bool,
        no_headers: bool,
        verbose: bool,
):
    """Показывает теги развертываний для компонента"""
    try:
        # Устанавливаем режим verbose для логирования
        set_verbose(verbose)

        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        # Используем сервис для получения данных в унифицированном формате
        svc = ComponentsService(api_client=api)

        # Используем callback для прогресс-бара (вызывается только при реальных запросах)
        from core.progress import create_progress_context

        with create_progress_context(
                "Получение информации о компонентах...",
        ) as progress_callback:
            component = svc.get(name, progress=progress_callback)

        if not component:
            console.print(f"Компонент '{name}' не найден")
            return

        # Извлекаем информацию о репозитории и образе (данные в snake_case)
        repo = component.get("repo", "")
        if not repo:
            console.print(
                f"[red]Не удалось получить информацию о репозитории для компонента {name}[/red]"
            )
            return

        # Получаем образ и тег (данные в snake_case)
        image_name = component.get("image", "")
        image_tag = component.get("tag", "")

        # Формируем image из TAG до __ включая __
        # ВАЖНО: API принимает формат ":<префикс_тега_до_двойного_подчеркивания_включая_его>"
        # Например: ":dev__" - это ПРАВИЛЬНЫЙ формат, НЕ МЕНЯТЬ!
        if "__" in image_tag:
            # Берем подстроку до __ включая __
            tag_part = image_tag.split("__")[0] + "__"
            # Используем только префикс тега с двоеточием
            search_image = f":{tag_part}"
        else:
            # Если нет __, используем полную версию тега
            tag_part = image_tag  # Для фильтрации
            search_image = f"{image_name}:{image_tag}"

        # Используем callback для прогресс-бара при получении тегов
        from core.progress import create_progress_context

        with create_progress_context(
                "Загрузка тегов развертываний...",
        ) as progress_callback:
            # Используем сервис для получения тегов
            tags = svc.get_deployment_tags(repo, search_image, no_cache, progress=progress_callback)

        if not tags:
            console.print(f"[yellow]Теги не найдены для {repo}/{search_image}[/yellow]")
            return

        # Фильтруем теги: исключаем тег, который точно соответствует поисковому запросу
        # Например, если ищем по ":dev__", то исключаем тег "dev__"
        filtered_tags = []
        for tag in tags:
            tag_name = tag["name"]
            # Если тег точно соответствует поисковому запросу (без дополнительной части), исключаем его
            if tag_name != tag_part:
                filtered_tags.append(tag)

        tags = filtered_tags

        if not tags:
            console.print(
                f"[yellow]Нет доступных тегов для {repo}/{search_image}[/yellow]"
            )
            return

        # Применяем лимит если указан
        if limit:
            tags = tags[:limit]

        # Подготавливаем данные для таблицы в стиле других list команд
        from utils.formatters import format_table, format_output

        data = []
        for tag in tags:
            data.append({"tag": tag["name"]})

        format_table(data, no_headers=no_headers)

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка API: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Неожиданная ошибка: {e}[/red]")
        raise click.Abort()


def _parse_component_tag(component_spec):
    """
    Парсит спецификацию компонента в формате 'component:tag' или 'component'

    Args:
        component_spec: Строка в формате 'component:tag' или 'component'

    Returns:
        tuple: (component_name, tag) где tag может быть None для 'latest'
    """
    if ":" in component_spec:
        component_name, tag = component_spec.rsplit(":", 1)
        return component_name.strip(), tag.strip()
    else:
        return component_spec.strip(), "latest"


def _load_upgrades_from_file(file_path):
    """
    Загружает список обновлений из файла в формате tab-separated

    Args:
        file_path: Путь к файлу

    Returns:
        list: Список кортежей (component_name, tag)
    """
    upgrades = []
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            for line_num, line in enumerate(f, 1):
                line = line.strip()

                # Пропускаем пустые строки и комментарии
                if not line or line.startswith("#"):
                    continue

                # Парсим tab-separated формат
                if "\t" in line:
                    parts = line.split("\t", 1)
                    if len(parts) == 2:
                        component_name = parts[0].strip()
                        tag = parts[1].strip()
                        upgrades.append((component_name, tag))
                    else:
                        console.print(
                            f"[yellow]Предупреждение: строка {line_num} в файле {file_path} имеет неверный формат[/yellow]"
                        )
                else:
                    console.print(
                        f"[yellow]Предупреждение: строка {line_num} в файле {file_path} не содержит табуляцию[/yellow]"
                    )

    except FileNotFoundError:
        console.print(f"[red]Файл {file_path} не найден[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Ошибка чтения файла {file_path}: {e}[/red]")
        raise click.Abort()

    return upgrades


def _extract_current_tag_from_component(component, svc: ComponentsService):
    """
    Извлекает текущий тег из компонента (из task, если доступен, иначе из component.image)

    Args:
        component: Словарь с информацией о компоненте
        svc: Сервис компонентов

    Returns:
        str: Текущий тег или 'latest' если не найден
    """
    try:
        # Сначала проверяем поле tag в унифицированном формате (snake_case)
        # Это поле уже содержит полный тег, если компонент получен через сервис
        if "tag" in component:
            tag = component.get("tag")
            if tag:
                return tag

        # Если нет поля tag, пытаемся извлечь из сырых данных
        # Используем методы сервиса для извлечения из task или image
        task_tag = svc.extract_tag_from_task(component)
        if task_tag:
            return task_tag
        return svc.extract_tag_from_component(component)
    except Exception:
        return "latest"


def _get_component_current_tag(api, component_name, no_cache=False):
    """
    Получает текущий тег компонента (из task, если доступен, иначе из component.image)

    Args:
        api: API клиент
        component_name: Имя компонента
        no_cache: Игнорировать кеш

    Returns:
        str: Текущий тег или None если компонент не найден
    """
    try:
        # Используем сервис для получения данных в унифицированном формате
        svc = ComponentsService(api_client=api)
        component = svc.get(component_name)
        if not component:
            return None
        # Возвращаем тег (данные в snake_case)
        tag = component.get("tag", "")
        return tag if tag else "latest"

    except Exception:
        return None


def _validate_tag_availability(api, component_name, target_tag, no_cache=False):
    """
    Проверяет доступность указанного тега в репозитории компонента

    Args:
        api: API клиент
        component_name: Имя компонента
        target_tag: Целевой тег для проверки
        no_cache: Не использовать кеш

    Returns:
        tuple: (is_available, error_message) где is_available - bool, error_message - str или None
    """
    try:
        # Используем сервис для получения данных в унифицированном формате
        svc = ComponentsService(api_client=api)
        component = svc.get(component_name)
        if not component:
            return False, "Компонент не найден"

        # Извлекаем информацию о репозитории и образе (данные в snake_case)
        repo = component.get("repo", "")
        if not repo:
            return False, "Репозиторий не найден"

        # Получаем образ и тег (данные в snake_case)
        image_name = component.get("image", "")
        image_tag = component.get("tag", "")
        if not image_tag:
            image_tag = "latest"

        # Для тегов с __ проверяем доступность через сервис
        if "__" in target_tag:
            # Формируем поисковый запрос по префиксу
            tag_part = target_tag.split("__")[0] + "__"
            search_image = f":{tag_part}"

            # Получаем список доступных тегов через сервис
            tags = svc.get_deployment_tags(repo, search_image, no_cache)
            if not tags:
                return False, f"Не удалось получить теги для {repo}/{search_image}"

            # Проверяем, есть ли целевой тег в списке
            available_tags = [tag["name"] for tag in tags]
            if target_tag not in available_tags:
                return False, f"Тег {target_tag} не найден в репозитории {repo}"

        # Для стабильных тегов (без __) считаем их всегда доступными
        # так как они не запрашиваются через API тегов

        return True, None

    except Exception as e:
        return False, f"Ошибка проверки тега: {str(e)}"


def _get_component_latest_tag(api, component_name, no_cache=False):
    """
    Получает последний доступный тег для компонента

    Args:
        api: API клиент
        component_name: Имя компонента

    Returns:
        str: Последний тег или None если не найден
    """
    try:
        # Используем сервис для получения данных в унифицированном формате
        svc = ComponentsService(api_client=api)
        component = svc.get(component_name)
        if not component:
            return None

        # Извлекаем информацию о репозитории и образе (данные в snake_case)
        repo = component.get("repo", "")
        if not repo:
            return None

        # Получаем образ и тег (данные в snake_case)
        image_name = component.get("image", "")
        image_tag = component.get("tag", "")
        if not image_tag:
            image_tag = "latest"

        # Формируем image из TAG до __ включая __
        # ВАЖНО: API принимает формат ":<префикс_тега_до_двойного_подчеркивания_включая_его>"
        # Например: ":dev__" - это ПРАВИЛЬНЫЙ формат, НЕ МЕНЯТЬ!
        if "__" in image_tag:
            tag_part = image_tag.split("__")[0] + "__"
            search_image = f":{tag_part}"
        else:
            # Для тегов без __ не запрашиваем список тегов - это стабильные версии
            return None

        # Получаем теги через сервис
        tags = svc.get_deployment_tags(repo, search_image, no_cache)

        if not tags:
            # Если теги не найдены, возвращаем None
            # Это означает, что компонент не может быть обновлен
            return None

        # Фильтруем теги: исключаем тег, который точно соответствует префиксу
        # Например, если ищем по ":dev__", то исключаем тег "dev__"
        filtered_tags = []
        for tag in tags:
            tag_name = tag["name"]
            if tag_name != tag_part:  # Исключаем префикс, а не полный тег
                filtered_tags.append(tag)

        if not filtered_tags:
            return None

        # Дополнительная проверка: если у компонента нет __ в теге,
        # но API возвращает теги с __, то это неправильный репозиторий
        if "__" not in image_tag and filtered_tags:
            # Проверяем, есть ли теги без __ в результате
            non_underscore_tags = [
                tag for tag in filtered_tags if "__" not in tag["name"]
            ]
            if not non_underscore_tags:
                # Если все теги содержат __, а у компонента нет, то это неправильный репозиторий
                return None
            # Используем только теги без __
            filtered_tags = non_underscore_tags

        # Возвращаем первый (самый новый) тег
        return filtered_tags[0]["name"]

    except Exception:
        return None


def _get_components_with_filters(api, filter_specs, no_cache=False):
    """
    Получает список компонентов с применением фильтров

    Args:
        api: API клиент
        filter_specs: Список спецификаций фильтров
        no_cache: Не использовать кеш

    Returns:
        Кортеж (upgrades, components_data) где:
        - upgrades: Список кортежей (component_name, tag)
        - components_data: Исходные данные компонентов
    """
    from rich.console import Console

    console = Console()

    # Используем ComponentsService для получения данных
    svc = ComponentsService(api_client=api)

    # Используем callback для прогресс-бара (вызывается только при реальных запросах)
    from core.progress import create_progress_context

    with create_progress_context(
            "Получение информации о компонентах...",
    ) as progress_callback:
        # Получаем компоненты через сервис (с фильтрами)
        components_list = svc.list(
            filters=list(filter_specs) if filter_specs else None,
            columns_spec=None,
            no_cache=no_cache,
            progress=progress_callback,
        )

        # Получаем исходные данные для components_data (используем тот же callback)
        deployments = api.get_components(no_cache=no_cache, progress=progress_callback)

    # Преобразуем в список кортежей (component_name, tag)
    upgrades = []
    for comp_data in components_list:
        name = comp_data.get("name", "")
        tag = comp_data.get("tag", "")
        if name:  # Пропускаем пустые имена
            upgrades.append((name, tag))

    return upgrades, deployments


def _validate_upgrade_plan(api, upgrades, no_cache=False, components_data=None, force=False):
    """
    Валидирует план обновления с оптимизацией запросов к API

    Args:
        api: API клиент
        force: Пропустить валидацию
        upgrades: Список кортежей (component_name, tag)
        no_cache: Не использовать кеш

    Returns:
        tuple: (valid_upgrades, invalid_upgrades)
    """
    valid_upgrades = []
    invalid_upgrades = []

    # Если force=True, пропускаем валидацию
    if force:
        verbose_print("Force mode: skipping validation")
        # Нужно получить components_index для формирования правильных данных
        if components_data is not None:
            all_deployments = components_data
        else:
            all_deployments = api.get_components(no_cache=no_cache)

        components_index = {}
        for group in all_deployments:
            stacks = group.get("stacks", [])
            for stack in stacks:
                stack_components = stack.get("components", [])
                for comp in stack_components:
                    comp_name = comp.get("name")
                    if comp_name:
                        result = comp.copy()
                        namespace = group.get("namespace")
                        stack_id = stack.get("stackId")
                        result["namespace"] = namespace
                        result["stack_id"] = f"{stack_id}@{namespace}"
                        components_index[comp_name] = result

        # Формируем valid_upgrades напрямую
        svc = ComponentsService(api_client=api)
        for component_name, target_tag in upgrades:
            if component_name in components_index:
                component_data = components_index[component_name]
                current_tag = _extract_current_tag_from_component(component_data, svc)
                component_namespace = component_data.get("namespace", "")
                valid_upgrades.append((component_name, target_tag, current_tag, component_namespace, component_data))

        verbose_print(f"Force mode: {len(valid_upgrades)} components prepared without validation")
        return valid_upgrades, []

    # Шаг 1: Получаем информацию о всех компонентах
    component_info = {}

    # Используем переданные данные или получаем их заново
    if components_data is not None:
        all_deployments = components_data
    else:
        all_deployments = api.get_components(no_cache=no_cache)

    # Создаем индекс компонентов по имени для быстрого поиска
    components_index = {}
    for group in all_deployments:
        stacks = group.get("stacks", [])
        for stack in stacks:
            stack_components = stack.get("components", [])
            for comp in stack_components:
                comp_name = comp.get("name")
                if comp_name:
                    # Добавляем дополнительную информацию о стеке и namespace
                    result = comp.copy()
                    namespace = group.get("namespace")
                    stack_id = stack.get("stackId")
                    result["namespace"] = namespace
                    result["stack_id"] = f"{stack_id}@{namespace}"
                    result["stack_version"] = stack.get("version")
                    result["stack_status"] = stack.get("status")
                    result["stack_repo"] = stack.get("repo")
                    result["stack_branch"] = stack.get("branch")
                    result["stack_commit"] = stack.get("commit")
                    result["stack_tag"] = stack.get("tag")
                    result["stack_timestamp"] = stack.get("timestamp")
                    components_index[comp_name] = result

    for component_name, target_tag in upgrades:
        # Ищем компонент в индексе (для получения сырых данных)
        component_raw = components_index.get(component_name)
        if not component_raw:
            invalid_upgrades.append((component_name, target_tag, "Компонент не найден"))
            continue

        # Получаем компонент через сервис для получения нормализованных данных с полным тегом
        svc = ComponentsService(api_client=api)
        component_normalized = svc.get(component_name, no_cache=no_cache)

        # Извлекаем текущий тег из нормализованных данных (содержат поле tag с полным тегом)
        if component_normalized and "tag" in component_normalized:
            current_tag = component_normalized.get("tag", "latest")
        else:
            # Fallback: извлекаем из сырых данных
            current_tag = _extract_current_tag_from_component(component_raw, svc)

        # Используем нормализованные данные если есть, иначе сырые
        component = component_normalized if component_normalized else component_raw

        component_info[component_name] = {
            "current_tag": current_tag,
            "target_tag": target_tag,
            "component": component,
        }

    # Шаг 2: Извлекаем уникальные пары repository + image для тегов с __
    unique_requests = set()
    component_requests = {}  # component_name -> (repository, image)

    for component_name, info in component_info.items():
        if info["target_tag"] != "latest":
            continue  # Пропускаем компоненты с конкретным тегом

        # Получаем информацию о компоненте для извлечения repository и image
        # Используем уже полученную информацию из component_info, если она есть
        component = info.get("component")
        if not component:
            # Используем сервис для получения данных в унифицированном формате
            svc = ComponentsService(api_client=api)
            component = svc.get(component_name)
            if not component:
                invalid_upgrades.append(
                    (component_name, info["target_tag"], "Компонент не найден")
                )
                continue

        # Получаем repo (данные в snake_case или сырые данные для обратной совместимости)
        repo = component.get("repo", "") if isinstance(component, dict) else component.get("repo", "")
        if not repo:
            invalid_upgrades.append(
                (component_name, info["target_tag"], "Репозиторий не найден")
            )
            continue

        # Получаем образ и тег (данные в snake_case или сырые данные для обратной совместимости)
        image_name = component.get("image", "") if isinstance(component, dict) else component.get("image", "")
        image_tag = component.get("tag", "") if isinstance(component, dict) else component.get("tag", "latest")
        if not image_tag:
            image_tag = "latest"

        # Проверяем, есть ли __ в теге
        if "__" in image_tag:
            tag_part = image_tag.split("__")[0] + "__"
            search_image = f":{tag_part}"
            unique_requests.add((repo, search_image))
            component_requests[component_name] = (repo, search_image)
        else:
            # Для тегов без __ не запрашиваем теги
            invalid_upgrades.append(
                (
                    component_name,
                    info["current_tag"],
                    "Стабильная версия - обновление недоступно",
                )
            )

    # Шаг 3: Получаем теги для каждой уникальной пары
    tags_cache = {}  # (repository, image) -> list of tags

    if unique_requests:
        from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
        from rich.console import Console

        console = Console()

        # Используем сервис для получения тегов
        svc = ComponentsService(api_client=api)

        with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                TimeElapsedColumn(),
                console=console,
        ) as progress:
            task = progress.add_task("Получение актуальных тегов...", total=None)

            for i, (repo, search_image) in enumerate(unique_requests, 1):
                # Формируем понятное описание для пользователя
                display_image = search_image.replace(":", "").replace(
                    "_", " "
                )  # Убираем двоеточие и подчеркивания для отображения
                display_repo = repo.replace(
                    "_", " "
                )  # Убираем подчеркивания для отображения
                progress.update(
                    task,
                    description=f"Получение актуального тега для {display_repo}/{display_image} ({i}/{len(unique_requests)})",
                )

                # Небольшая задержка для видимости прогресса
                import time

                time.sleep(0.1)

                # Создаем callback, который обновляет общий прогресс
                # Callback от сервиса будет вызван только при реальных запросах
                from core.progress import RichProgressCallback
                step_info = f"{i}/{len(unique_requests)}"
                progress_callback = RichProgressCallback(console, progress, task, step_info=step_info)

                # Используем сервис с callback (callback вызовется автоматически при реальном запросе)
                tags = svc.get_deployment_tags(repo, search_image, no_cache=no_cache, progress=progress_callback)
                tags_cache[(repo, search_image)] = tags

    # Шаг 4: Применяем результаты к компонентам
    for component_name, info in component_info.items():
        if info["target_tag"] != "latest":
            # Для компонентов с конкретным тегом проверяем доступность тега
            if info["current_tag"] != info["target_tag"]:
                # Проверяем доступность целевого тега в репозитории
                is_available, error_msg = _validate_tag_availability(
                    api, component_name, info["target_tag"], no_cache
                )
                if is_available:
                    # Получаем namespace из информации о компоненте
                    component_namespace = info["component"].get("namespace", "")
                    valid_upgrades.append(
                        (
                            component_name,
                            info["target_tag"],
                            info["current_tag"],
                            component_namespace,
                            info["component"],
                        )
                    )
                else:
                    invalid_upgrades.append(
                        (component_name, info["target_tag"], error_msg)
                    )
            continue

        if component_name not in component_requests:
            continue  # Уже обработан выше

        repo, search_image = component_requests[component_name]
        tags = tags_cache.get((repo, search_image), [])

        if not tags:
            invalid_upgrades.append(
                (
                    component_name,
                    info["current_tag"],
                    "Не удалось получить последний тег",
                )
            )
            continue

        # Ищем последний тег (исключая префикс)
        current_tag = info["current_tag"]
        latest_tag = None

        # Определяем префикс для исключения
        if "__" in current_tag:
            tag_part = current_tag.split("__")[0] + "__"
        else:
            tag_part = current_tag

        for tag in tags:
            tag_name = tag.get("name", tag.get("tag", ""))
            if tag_name != tag_part:  # Исключаем префикс, а не полный тег
                latest_tag = tag_name
                break

        if latest_tag is None:
            invalid_upgrades.append(
                (component_name, info["current_tag"], "Нет доступных обновлений")
            )
            continue

        # Проверяем, что теги разные
        if current_tag == latest_tag:
            continue

        # Получаем namespace из информации о компоненте
        component_namespace = info["component"].get("namespace", "")
        valid_upgrades.append(
            (
                component_name,
                latest_tag,
                current_tag,
                component_namespace,
                info["component"],
            )
        )

    return valid_upgrades, invalid_upgrades


@cli.command("upgrade")
@click.argument("components", nargs=-1, metavar="[COMPONENT:TAG]...", required=False)
@click.option(
    "--from-file",
    "upgrade_file",
    help="Загрузить список обновлений из файла (tab-separated)",
)
@click.option(
    "--check", is_flag=True, help="Показать доступные обновления без выполнения"
)
@click.option("--dry-run", is_flag=True, help="Предварительный просмотр без выполнения")
@click.option("--all", is_flag=True, help="Обновить все компоненты до последних тегов")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.option(
    "--filter",
    "filter_spec",
    multiple=True,
    help="Фильтр компонентов. Формат: COLUMN=value",
)
@click.option(
    "--to-tag",
    help="Обновить все отфильтрованные компоненты до указанного тега (с проверкой доступности)",
)
@click.option(
    "--confirm", is_flag=True, help="Подтвердить обновления без интерактивного запроса"
)
@click.option("--force", is_flag=True, help="Принудительное обновление без проверки доступности тега")
@click.option("--no-cache", is_flag=True, help="Не использовать кеш при проверке тегов")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option(
    "--export-current", is_flag=True, help="Экспортировать текущие версии в stdout"
)
@click.option(
    "--export-latest", is_flag=True, help="Экспортировать последние версии в stdout"
)
@click.option(
    "--export-upgradable",
    is_flag=True,
    help="Экспортировать компоненты для обновления в stdout",
)
@click.pass_context
def upgrade_components(
        ctx,
        components,
        upgrade_file,
        check,
        dry_run,
        all,
        filter_spec,
        to_tag,
        confirm,
        force,
        no_cache,
        no_headers,
        export_current,
        export_latest,
        export_upgradable,
        verbose,
):
    """Обновляет компоненты до указанных тегов

    Примеры:
      imgctl components upgrade web-api-staging                    # Обновить до последнего тега
      imgctl components upgrade web-api-staging:v1.2.3            # Обновить до конкретного тега
      imgctl components upgrade web-api-staging db-staging:latest # Несколько компонентов
      imgctl components upgrade --from-file upgrades.txt          # Из файла
      imgctl components upgrade --all                             # Все компоненты
      imgctl components upgrade --filter NAME~^oms-api- --to-tag dev__2025_01_15_10_00-abc123  # Обновить все oms-api компоненты до указанного тега
      imgctl components upgrade --filter NAMESPACE=staging --to-tag latest  # Обновить все компоненты в namespace staging
      imgctl components upgrade --filter STATUS=RUNNING --to-tag dev__2025_01_15_10_00-abc123  # Обновить все запущенные компоненты
      imgctl components upgrade --check                           # Показать доступные обновления
      imgctl components upgrade --dry-run                         # Предварительный просмотр без выполнения
    """
    try:
        # Устанавливаем режим verbose для логирования
        set_verbose(verbose)

        # Создаем API клиент с нужными параметрами
        api = _get_api_client(verbose=verbose, ctx=ctx)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        # Обработка режимов экспорта
        if export_current or export_latest or export_upgradable:
            # Определяем список компонентов для экспорта
            components_list = []

            if components:
                # Если указаны компоненты в аргументах, используем их
                for component_spec in components:
                    component_name, _ = _parse_component_tag(component_spec)
                    # Получаем текущий тег через сервис
                    svc = ComponentsService(api_client=api)
                    component = svc.get(component_name, no_cache=no_cache)
                    if component:
                        current_tag = component.get("tag", "latest")
                        components_list.append((component_name, current_tag))
            elif filter_spec:
                # Если есть фильтры, используем их
                components_list, _ = _get_components_with_filters(
                    api, list(filter_spec), no_cache
                )
            else:
                # Иначе используем все компоненты
                components_list, _ = _get_components_with_filters(
                    api, None, no_cache
                )

            if export_current:
                # Экспортируем текущие версии
                for component_name, current_tag in components_list:
                    print(f"{component_name}\t{current_tag}")
                return

            elif export_latest:
                # Экспортируем последние версии
                for component_name, _ in components_list:
                    latest_tag = _get_component_latest_tag(
                        api, component_name, no_cache
                    )
                    if latest_tag:
                        print(f"{component_name}\t{latest_tag}")
                    else:
                        print(f"{component_name}\t")
                return

            elif export_upgradable:
                # Экспортируем компоненты для обновления
                # Формируем список upgrades для валидации с прогресс-баром
                upgrades_for_validation = [(name, "latest") for name, _ in components_list]

                # Валидируем план обновления (это покажет прогресс)
                valid_upgrades, invalid_upgrades = _validate_upgrade_plan(
                    api, upgrades_for_validation, no_cache, None, force
                )

                # Экспортируем валидные обновления
                for component_name, latest_tag, current_tag, _, _ in valid_upgrades:
                    print(f"{component_name}\t{latest_tag}")
                return

        # Определяем список обновлений
        upgrades = []

        if upgrade_file:
            # Загружаем из файла
            console.print(
                f"[blue]Загрузка обновлений из файла {upgrade_file}...[/blue]"
            )
            upgrades = _load_upgrades_from_file(upgrade_file)
        elif all or (not components and not filter_spec and not upgrade_file):
            # Получаем все компоненты без фильтров
            upgrades, components_data = _get_components_with_filters(
                api, None, no_cache
            )

            # Заменяем теги на 'latest' для обновления до последних версий
            upgrades = [(name, "latest") for name, _ in upgrades]
        elif filter_spec:
            # Получаем компоненты с применением фильтров (используем TTL кеша)
            upgrades, components_data = _get_components_with_filters(
                api, list(filter_spec), no_cache
            )

            if to_tag:
                # Обновляем все отфильтрованные компоненты до указанного тега

                # Пропускаем валидацию если используется --force
                if force:
                    console.print("[yellow]Флаг --force активен: проверка доступности тегов пропущена[/yellow]")
                    upgrades = [(name, to_tag) for name, _ in upgrades]
                    verbose_print(f"Force mode: {len(upgrades)} components will be upgraded without validation")
                else:
                    # Проверяем доступность тега для каждого компонента с прогресс-баром
                    original_count = len(upgrades)
                    validated_upgrades = []

                    with Progress(
                            SpinnerColumn(),
                            TextColumn("[progress.description]{task.description}"),
                            console=console,
                    ) as progress:
                        task = progress.add_task(
                            f"Проверка доступности тега '{to_tag}' для {original_count} компонентов...",
                            total=original_count
                        )

                        for name, _ in upgrades:
                            is_available, error_msg = _validate_tag_availability(
                                api, name, to_tag, no_cache
                            )
                            if is_available:
                                validated_upgrades.append((name, to_tag))
                            else:
                                console.print(f"[yellow]Пропуск {name}: {error_msg}[/yellow]")
                            progress.advance(task)

                    upgrades = validated_upgrades

                    if len(validated_upgrades) < original_count:
                        console.print(
                            f"[blue]Валидных обновлений: {len(validated_upgrades)} из {original_count}[/blue]"
                        )
            else:
                # Заменяем теги на 'latest' для обновления до последних версий
                upgrades = [(name, "latest") for name, _ in upgrades]
        elif components:
            # Парсим аргументы командной строки (используем TTL кеша)
            for component_spec in components:
                component_name, tag = _parse_component_tag(component_spec)
                upgrades.append((component_name, tag))

            # Получаем данные компонентов для валидации
            _, components_data = _get_components_with_filters(api, None, no_cache)

        if not upgrades:
            console.print("[yellow]Нет компонентов для обновления[/yellow]")
            return

        # Валидируем план обновления
        valid_upgrades, invalid_upgrades = _validate_upgrade_plan(
            api, upgrades, no_cache, components_data, force
        )

        # Показываем невалидные обновления
        if invalid_upgrades:
            console.print("[red]Найдены ошибки в плане обновления:[/red]")
            for component_name, target_tag, error in invalid_upgrades:
                # Определяем namespace для подсветки
                component_namespace = None
                if components_data:
                    for group in components_data:
                        for stack in group.get("stacks", []):
                            for comp in stack.get("components", []):
                                if comp.get("name") == component_name:
                                    component_namespace = group.get("namespace", "")
                                    break

                from utils.highlighter import Highlighter
                highlighted_name = (
                    Highlighter.highlight_component_name(component_name, component_namespace)
                    if component_namespace
                    else component_name
                )
                highlighted_tag = Highlighter.highlight_tag(target_tag)
                console.print(f"  ❌ {highlighted_name}:{highlighted_tag} - {error}")
            console.print()

        if not valid_upgrades:
            console.print("[red]Нет валидных обновлений для выполнения[/red]")
            return

        # Показываем план обновления
        console.print(
            f"[green]План обновления ({len(valid_upgrades)} компонентов):[/green]"
        )
        for (
                component_name,
                target_tag,
                current_tag,
                component_namespace,
                component_data,
        ) in valid_upgrades:
            if current_tag == target_tag:
                # Создаем Text объект для правильной раскраски
                from rich.text import Text

                text = Text()
                text.append("  ")

                # Подсвечиваем имя компонента
                if "-" in component_name and component_namespace:
                    namespace_suffix = f"-{component_namespace}"
                    if component_name.endswith(namespace_suffix):
                        component_part = component_name[: -len(namespace_suffix)]
                        text.append(component_part)
                        text.append(namespace_suffix, style="dim")
                    else:
                        text.append(component_name)
                else:
                    text.append(component_name)

                text.append(": ")
                text.append("уже актуален", style="green")
                console.print(text)
            else:
                from rich.text import Text

                text = Text()
                text.append("  ")

                # Подсвечиваем имя компонента
                if "-" in component_name and component_namespace:
                    namespace_suffix = f"-{component_namespace}"
                    if component_name.endswith(namespace_suffix):
                        component_part = component_name[: -len(namespace_suffix)]
                        text.append(component_part)
                        text.append(namespace_suffix, style="dim")
                    else:
                        text.append(component_name)
                else:
                    text.append(component_name)

                text.append(": ")
                text.append(current_tag, style="yellow")
                text.append(" → ")
                text.append(target_tag, style="green")
                console.print(text)

        # Если это режим проверки или dry-run, выходим
        if check or dry_run:
            if check:
                console.print(
                    "\n[blue]Режим проверки - обновления не выполняются[/blue]"
                )
            else:
                console.print(
                    "\n[blue]Dry-run режим - обновления не выполняются[/blue]"
                )
            return

        # Подтверждение
        if not confirm:
            if not click.confirm(
                    f"\nВыполнить обновление {len(valid_upgrades)} компонентов?"
            ):
                console.print("[yellow]Обновление отменено[/yellow]")
                return

        # Выполняем обновления с прогресс-барами
        console.print(f"\n[green]Выполнение обновлений...[/green]")
        success_count = 0
        error_count = 0

        # Выполняем обновления с прогресс-барами
        with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                TimeElapsedColumn(),
                console=console,
                transient=False,
        ) as progress:

            for (
                    component_name,
                    target_tag,
                    current_tag,
                    component_namespace,
                    component_data,
            ) in valid_upgrades:
                # DEBUG: выводим информацию о всех компонентах для обновления
                verbose_print(f"Component: {component_name}, current: {current_tag}, target: {target_tag}")

                # С --force пропускаем проверку "уже актуален"
                if current_tag == target_tag and not force:
                    verbose_print(f"  Skipping {component_name} - already up to date")
                    continue  # Пропускаем уже актуальные (если не force)

                from utils.highlighter import Highlighter
                highlighted_name = Highlighter.highlight_component_name(
                    component_name, component_namespace
                )

                # Создаем описание для прогресса с подсветкой тегов
                description = f"{highlighted_name}: [yellow]{current_tag}[/yellow] → [green]{target_tag}[/green]"

                # Создаем задачу с анимацией и временем
                task_id = progress.add_task(description, total=100)

                try:
                    # Используем уже полученные данные о компоненте
                    component = component_data

                    # Получаем образ и тег (данные в snake_case или сырые данные для обратной совместимости)
                    # component_data из сервиса в snake_case, из старого кода может быть в сыром формате
                    image_name = component.get("image", "")
                    if not image_name:
                        # Если нет в snake_case, извлекаем из сырых данных (для обратной совместимости)
                        image = component.get("image", "")
                        child_container = component.get("childContainer")
                        if child_container and child_container.get("image"):
                            image = child_container.get("image")
                        # Извлекаем имя образа без тега
                        if ":" in image:
                            image_name, _ = image.rsplit(":", 1)
                        else:
                            image_name = image
                    # image в snake_case уже без тега

                    # Формируем новый образ с целевым тегом
                    new_image = f"{image_name}:{target_tag}"

                    # Выполняем обновление
                    # Извлекаем stack (данные в snake_case или сырые данные для обратной совместимости)
                    stack = component.get("stack", "")
                    if stack:
                        stack_name = stack.split("@")[0] if "@" in stack else stack
                    else:
                        # Пробуем получить из сырых данных для обратной совместимости
                        stack_id = component.get("stack_id", "")
                        stack_name = stack_id.split("@")[0] if "@" in stack_id and stack_id else ""

                    verbose_print(f"Upgrading {component_name}: {current_tag} -> {target_tag}")
                    verbose_print(f"  namespace: {component_namespace}, stack: {stack_name}")
                    verbose_print(f"  new image: {new_image}")

                    svc = ComponentsService(api_client=api)
                    svc.update(
                        component_name, new_image, component_namespace, stack_name, component_data
                    )

                    # Завершаем прогресс
                    progress.update(task_id, advance=100)
                    progress.remove_task(task_id)

                    success_count += 1

                except Exception as e:
                    # Завершаем прогресс
                    progress.update(task_id, advance=100)
                    progress.remove_task(task_id)

                    error_count += 1
                    # Выводим сообщение об ошибке
                    console.print(
                        f"❌ {highlighted_name}: [yellow]{current_tag}[/yellow] → [green]{target_tag}[/green]: {str(e)[:50]}..."
                    )
                    continue

                # Выводим финальное сообщение об успехе
                console.print(
                    f"✅ {highlighted_name}: [yellow]{current_tag}[/yellow] → [green]{target_tag}[/green]"
                )

        # Выводим итоговую статистику
        if success_count > 0 or error_count > 0:
            console.print()
            if success_count > 0:
                console.print(
                    f"[green]Успешно обновлено: {success_count} компонентов[/green]"
                )
            if error_count > 0:
                console.print(
                    f"[red]Ошибок обновления: {error_count} компонентов[/red]"
                )

    except ImagenariumAPIError as e:
        console.print(f"[red]Ошибка API: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Неожиданная ошибка: {e}[/red]")
        raise click.Abort()



"""
Класс для подсветки статусов, имен компонентов и других значений с цветовым кодированием
"""

from typing import Any


class Highlighter:
    """Класс для стилизации статусов и значений статусов с Rich markup"""

    # Значения для зеленого цвета (успешные/рабочие)
    SUCCESS = {
        "RUNNING", "ACTIVE", "READY", "HEALTHY", "DEPLOYED", "AVAILABLE"
    }

    # Значения для красного цвета (ошибки/остановленные)
    ERROR = {
        "STOPPED", "INACTIVE", "TERMINATED", "FAILED", "ERROR",
        "OFF", "CANCELED", "UNHEALTHY", "DOWN", "DRAIN", "DRAINING", "UNAVAILABLE"
    }

    # Значения для желтого цвета (ожидание/предупреждение)
    WARNING = {
        "PENDING", "WAITING", "STARTING", "BROKEN", "PAUSE", "PAUSED"
    }

    @classmethod
    def highlight(cls, value: str, check_partial: bool = True, preserve_markup: bool = True) -> str:
        """
        Подсвечивает значение статуса или доступности цветом

        Args:
            value: Значение для подсветки
            check_partial: Если True, проверяет частичные статусы (PARTIAL_*).
                          По умолчанию True для статусов, False для availability
            preserve_markup: Если True, не перезаписывает существующий Rich markup

        Returns:
            Строка с Rich markup
        """
        if not value:
            return value

        # Проверяем, есть ли уже Rich markup
        if preserve_markup and "[" in value and "]" in value:
            return value

        value_upper = value.upper()

        # Частичные статусы (например, PARTIAL_RUNNING) - только для статусов
        if check_partial and value_upper.startswith("PARTIAL"):
            return f"[bold orange3]{value}[/bold orange3]"

        # Успешные значения
        if value_upper in cls.SUCCESS:
            return f"[bold green]{value}[/bold green]"

        # Ошибочные значения
        if value_upper in cls.ERROR:
            return f"[bold red]{value}[/bold red]"

        # Предупреждающие значения
        if value_upper in cls.WARNING:
            return f"[bold yellow]{value}[/bold yellow]"

        # По умолчанию без стилизации
        return value

    @classmethod
    def highlight_component_name(cls, name: str, namespace: str) -> str:
        """
        Подсвечивает имя компонента: все что до namespace обычным белым, namespace серым

        Args:
            name: Имя компонента (может содержать Rich markup от фильтров, который будет удален)
            namespace: Namespace компонента

        Returns:
            Строка с Rich markup
        """
        import re

        # Убираем существующий Rich markup для поиска
        clean_name = re.sub(r'\[[^\]]*\]', '', name)

        if "-" in clean_name and namespace:
            # Ищем namespace в конце имени компонента
            namespace_suffix = f"-{namespace}"
            if clean_name.endswith(namespace_suffix):
                component_part = clean_name[: -len(namespace_suffix)]
                namespace_part = namespace_suffix
                # Просто применяем подсветку к чистому тексту
                return f"{component_part}[dim]{namespace_part}[/dim]"
            else:
                # Если namespace не найден в конце, ищем его в середине
                # Например: oms-suzlet-suz5 с namespace suzlet-suz5
                if namespace in clean_name:
                    # Находим позицию namespace в имени
                    namespace_pos = clean_name.find(namespace)
                    if namespace_pos > 0 and clean_name[namespace_pos - 1] == "-":
                        # Namespace найден после дефиса
                        component_part = clean_name[: namespace_pos - 1]
                        namespace_part = clean_name[namespace_pos - 1:]
                        return f"{component_part}[dim]{namespace_part}[/dim]"

        return clean_name

    @classmethod
    def highlight_tag(cls, tag: str) -> str:
        """
        Подсвечивает тег: строка до __ жирным белым, остальное обычным белым

        Args:
            tag: Тег для подсветки

        Returns:
            Строка с Rich markup
        """
        if "__" in tag:
            # Разделяем на часть до __ и после
            parts = tag.split("__", 1)
            if len(parts) == 2:
                before_part = parts[0]
                after_part = "__" + parts[1]
                return f"[bold white]{before_part}[/bold white][white]{after_part}[/white]"

        return tag

    @classmethod
    def highlight_value(cls, value: Any) -> str:
        """
        Форматирует значение с выделением пробелов в начале/конце красным цветом
        Используется для переменных окружения и параметров стеков

        Args:
            value: Значение для форматирования

        Returns:
            Строка с Rich markup, где пробелы в начале/конце выделены красным
        """

        # Если значение не строка, применяем форматирование по типу
        if not isinstance(value, str):
            # Boolean значения - yellow
            if isinstance(value, bool):
                return f"[yellow]{str(value).lower()}[/yellow]"

            # Числа - cyan
            if isinstance(value, (int, float)):
                return f"[cyan]{value}[/cyan]"

            return str(value)

        str_value = str(value)

        # Проверяем начальные и конечные пробелы
        leading_spaces = len(str_value) - len(str_value.lstrip())
        trailing_spaces = len(str_value) - len(str_value.rstrip())

        if leading_spaces > 0 or trailing_spaces > 0:
            # Разделяем на части: начальные пробелы, основной текст, конечные пробелы
            main_part = str_value.strip()
            leading_part = str_value[:leading_spaces]
            trailing_part = str_value[-trailing_spaces:] if trailing_spaces > 0 else ""

            # Форматируем основной текст по типу
            formatted_main = cls._highlight_value(main_part)

            # Собираем результат с красными пробелами (красный фон с белым текстом для лучшей видимости)
            result = f"[bold white on red]{leading_part}[/bold white on red]"
            result += formatted_main
            if trailing_part:
                result += f"[bold white on red]{trailing_part}[/bold white on red]"

            return result

        # Если пробелов нет, применяем форматирование по типу
        return cls._highlight_value(str_value)

    @classmethod
    def _highlight_value(cls, str_value: str) -> str:
        """
        Вспомогательный метод для форматирования содержимого значения по типу
        """
        import re

        str_lower = str_value.strip().lower()

        # Проверяем boolean значения в строках (true/false, y/n)
        if str_lower in ("true", "false", "y", "n"):
            return f"[yellow]{str_value}[/yellow]"

        # Проверяем, является ли строка URL/ссылкой
        url_pattern = re.compile(
            r'^https?://'  # http:// или https://
            r'[^\s/$.?#].[^\s]*$',  # остальная часть URL
            re.IGNORECASE
        )

        if url_pattern.match(str_value.strip()):
            # Ссылки - cyan
            return f"[cyan]{str_value}[/cyan]"

        # Все остальное (строки) - без раскраски
        return str_value



"""
HTTP клиент с централизованным кэшированием

Обеспечивает единую точку для всех HTTP запросов с автоматическим кэшированием
на основе URL и правил кэширования.
"""

import json
from typing import Dict, Optional, Any

import requests

from .cache_manager import CacheManager

# Импорт для callback прогресса
try:
    from .progress import ProgressCallback, NullProgressCallback
except ImportError:
    # Fallback если progress модуль еще не создан
    from typing import TYPE_CHECKING

    if TYPE_CHECKING:
        from .progress import ProgressCallback, NullProgressCallback
    else:
        ProgressCallback = Any


        # Создаем простой fallback класс
        class _NullProgressCallbackFallback:
            def on_request_start(self, description: str): pass

            def on_request_end(self): pass


        NullProgressCallback = _NullProgressCallbackFallback

# Импортируем verbose_print для единого verbose режима
try:
    from utils.verbose import verbose_print
except ImportError:
    # Fallback для случаев когда utils недоступен
    def verbose_print(msg: str):
        pass


class HTTPClient:
    """HTTP клиент с централизованным кэшированием"""

    def __init__(
            self,
            base_url: str,
            cache_manager: Optional[CacheManager] = None,
            verbose: bool = False,
    ):
        """
        Инициализация HTTP клиента

        Args:
            base_url: Базовый URL API
            cache_manager: Менеджер кэша (если None, кэширование отключено)
            verbose: Параметр игнорируется, оставлен для совместимости
        """
        self.base_url = base_url.rstrip("/")
        self.cache = cache_manager
        self.session = requests.Session()

        # Настройки сессии
        self.session.headers.update(
            {
                "User-Agent": "imgctl/1.0",
                "Accept": "application/json",
                "Content-Type": "application/json",
            }
        )

    def get(
            self,
            endpoint: str,
            params: Optional[Dict] = None,
            no_cache: bool = False,
            ttl: Optional[int] = None,
            progress: Optional[ProgressCallback] = None,
            **kwargs,
    ) -> requests.Response:
        """
        Выполняет GET запрос с кэшированием

        Args:
            endpoint: Эндпоинт API
            params: Параметры запроса
            no_cache: Не использовать кэш
            ttl: Время жизни кэша в секундах (переопределяет правило)
            progress: Callback для отображения прогресса (вызывается только при реальных запросах)
            **kwargs: Дополнительные параметры для requests

        Returns:
            Response объект
        """
        return self._make_request(
            "GET", endpoint, params=params, no_cache=no_cache, ttl=ttl, progress=progress, **kwargs
        )

    def post(
            self, endpoint: str, data: Optional[Dict] = None, **kwargs
    ) -> requests.Response:
        """
        Выполняет POST запрос

        Args:
            endpoint: Эндпоинт API
            data: Данные для отправки
            **kwargs: Дополнительные параметры для requests

        Returns:
            Response объект
        """
        return self._make_request("POST", endpoint, json=data, **kwargs)

    def put(
            self, endpoint: str, data: Optional[Dict] = None, **kwargs
    ) -> requests.Response:
        """
        Выполняет PUT запрос

        Args:
            endpoint: Эндпоинт API
            data: Данные для отправки
            **kwargs: Дополнительные параметры для requests

        Returns:
            Response объект
        """
        return self._make_request("PUT", endpoint, json=data, **kwargs)

    def delete(self, endpoint: str, **kwargs) -> requests.Response:
        """
        Выполняет DELETE запрос

        Args:
            endpoint: Эндпоинт API
            **kwargs: Дополнительные параметры для requests

        Returns:
            Response объект
        """
        return self._make_request("DELETE", endpoint, **kwargs)

    def _make_request(
            self,
            method: str,
            endpoint: str,
            no_cache: bool = False,
            ttl: Optional[int] = None,
            progress: Optional[ProgressCallback] = None,
            **kwargs,
    ) -> requests.Response:
        """
        Выполняет HTTP запрос с централизованным кэшированием

        Args:
            method: HTTP метод
            endpoint: Эндпоинт API
            no_cache: Не использовать кэш
            ttl: Время жизни кэша в секундах (переопределяет правило)
            progress: Callback для отображения прогресса (вызывается только при реальных запросах)
            **kwargs: Дополнительные параметры для requests
        """
        url = f"{self.base_url}{endpoint}"

        # Инициализируем callback если не передан
        if progress is None:
            progress = NullProgressCallback()

        # Для GET запросов проверяем кэш (если не отключен)
        if method.upper() == "GET" and not no_cache and self.cache:
            try:
                params = kwargs.get("params")
                # Проверяем кеш - логика бесконечного TTL обрабатывается в cache_manager
                cached_data = self.cache.get_cached(url, params)

                if cached_data:
                    data, etag = cached_data

                    # Логируем использование кэша
                    full_url = url
                    if params:
                        params_str = "&".join(
                            f"{k}={v}" for k, v in sorted(params.items())
                        )
                        full_url = f"{url}?{params_str}"

                    verbose_print(f"HTTP GET {full_url} (from cache)")

                    # Callback НЕ вызывается - данные из кеша, запрос мгновенный

                    # Создаем mock response с кэшированными данными
                    response = requests.Response()
                    response.status_code = 200
                    response._content = (
                        json.dumps(data).encode()
                        if isinstance(data, (dict, list))
                        else str(data).encode()
                    )
                    response.headers = {
                        "Content-Type": "application/json",
                        "ETag": etag,
                    }
                    return response
            except Exception:
                pass  # Игнорируем ошибки кэширования

        # Данные не в кеше - будет реальный запрос, вызываем callback
        try:
            # Формируем описание для прогресса
            # Для /deployments/tags используем информацию из параметров
            if endpoint == "/deployments/tags" and kwargs.get("params"):
                params = kwargs.get("params")
                repository = params.get("repository", "")
                image = params.get("image", "")
                # Убираем двоеточие из начала image если есть
                display_image = image.lstrip(":")
                # Убираем __ из конца если есть
                display_image = display_image.rstrip("__")
                description = f"Получение актуального тега для {repository}/{display_image}"
            else:
                description = f"Запрос {method.upper()} {endpoint}"
            progress.on_request_start(description)

            try:
                # Выполняем реальный запрос
                response = self.session.request(method, url, **kwargs)
            finally:
                # Всегда вызываем on_request_end, даже при ошибке
                progress.on_request_end()

            # Для успешных GET запросов сохраняем в кэш
            # no_cache влияет только на чтение из кеша, но не на запись
            if method.upper() == "GET" and self.cache and response.status_code == 200:
                try:
                    data = (
                        response.json()
                        if response.headers.get("Content-Type", "").startswith(
                            "application/json"
                        )
                        else response.text
                    )
                    etag = response.headers.get("ETag", "no-etag")

                    # Логируем сохранение в кэш
                    full_url = url
                    if kwargs.get("params"):
                        params_str = "&".join(
                            f"{k}={v}" for k, v in sorted(kwargs["params"].items())
                        )
                        full_url = f"{url}?{params_str}"
                    verbose_print(f"HTTP GET {full_url} (saved to cache)")

                    # Сохраняем в кэш с централизованными правилами
                    params = kwargs.get("params")
                    self.cache.set_cached(url, params, etag, data, ttl)
                except Exception:
                    pass  # Игнорируем ошибки кэширования

            response.raise_for_status()
            return response

        except requests.exceptions.RequestException as e:
            raise Exception(f"Ошибка HTTP запроса: {e}")

    def invalidate_cache(self, endpoint: str, params: Optional[Dict] = None) -> None:
        """
        Инвалидирует кэш для конкретного эндпоинта

        Args:
            endpoint: Эндпоинт API
            params: Параметры запроса
        """
        if self.cache:
            url = f"{self.base_url}{endpoint}"
            self.cache.invalidate(url, params)

    def invalidate_cache_pattern(self, pattern: str) -> None:
        """
        Инвалидирует кэш по паттерну

        Args:
            pattern: Паттерн для поиска в URL
        """
        if self.cache:
            self.cache.invalidate_pattern(pattern)

    def clear_cache(self) -> None:
        """Очищает весь кэш"""
        if self.cache:
            self.cache.clear()



"""
Утилиты imgctl

Содержит вспомогательные функции и утилиты.
"""

from .formatters import format_table, format_table, format_json, format_yaml
from .validators import validate_url, validate_namespace, validate_stack_id

__all__ = [
    "format_table",
    "format_table",
    "format_json",
    "format_yaml",
    "validate_url",
    "validate_namespace",
    "validate_stack_id",
]



"""
Команды для управления серверами
"""

import os

import click
from rich.console import Console
from rich.table import Table

from core.api_client import ImagenariumAPIError
from core.cache_manager import CacheManager
from core.services.servers_service import ServersService
from utils.formatters import format_table, format_output, show_console_header

console = Console()


def _get_cache_manager() -> CacheManager:
    """Создает CacheManager для управления настройками TTL"""
    return CacheManager()


@click.group()
def cli():
    """Управление серверами и подключениями к «Imagenarium»"""
    pass


@cli.command("list")
@click.option(
    "--show-passwords", is_flag=True, help="Показывать пароли (не рекомендуется)"
)
@click.option(
    "--columns",
    help='Столбцы для отображения. Форматы: "NAME,URL" или "+DESCRIPTION,-DEFAULT". Доступные: NAME, URL, USERNAME, DESCRIPTION, DEFAULT, PASSWORD',
)
@click.option(
    "--filter",
    multiple=True,
    help="Фильтрация данных. Формат: COLUMN=value, COLUMN!=value, COLUMN~pattern. Примеры: NAME=dev, URL~harbor, DEFAULT=✓",
)
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option(
    "--output",
    "-o",
    help='Формат вывода данных: json, yaml, tsv, или template (например: "{NAME} - {URL}")',
)
@click.pass_context
def list_servers(ctx, show_passwords, columns, filter, no_headers, output):
    """Показывает список серверов"""
    try:
        svc = ServersService()

        # Получаем данные из сервиса (в snake_case)
        # columns_spec передается в list, где он парсится и применяется
        data = svc.list(
            filters=list(filter) if filter else None,
            columns_spec=columns,
        )

        if not data:
            console.print(
                "[yellow]Серверы не найдены. Добавьте сервер командой 'servers add'[/yellow]"
            )
            return

        # Опционально подгружаем пароль из сервиса, если запрошено
        if show_passwords:
            svc = ServersService()
            for row in data:
                server_name = row.get("name", "")
                if server_name:
                    password = svc._get_password(server_name)
                    row["password"] = password or ""

        # Определяем столбцы для вывода (данные уже отфильтрованы в list)
        from utils.formatters import parse_filters

        # Если columns указан, используем столбцы из данных, иначе default
        if columns and data:
            columns_to_show = list(data[0].keys())
            # Добавляем password если show_passwords и его еще нет
            if show_passwords and "password" not in columns_to_show:
                columns_to_show.append("password")
        else:
            columns_to_show = svc.get_default_columns()
            if show_passwords and "password" not in columns_to_show:
                columns_to_show.append("password")

        # Данные уже отфильтрованы по столбцам в list, но нужно убедиться что password есть если нужно
        filtered_data = []
        for row in data:
            filtered_row = {col: row.get(col, "") for col in columns_to_show}
            if filtered_row:
                filtered_data.append(filtered_row)

        # Отображаем данные в указанном формате
        if not output or output == "table":
            highlight_filters = parse_filters(list(filter)) if filter else None
            format_table(
                filtered_data,
                columns=columns_to_show,
                no_headers=no_headers,
                highlight_filters=highlight_filters,
            )
        else:
            format_output(filtered_data, output, columns=columns_to_show)

    except Exception as e:
        console.print(f"[red]Ошибка получения списка серверов: {e}[/red]")
        raise click.Abort()


@cli.command("add")
@click.argument("name")
@click.option("--url", required=True, help="URL сервера")
@click.option("--username", required=True, help="Имя пользователя")
@click.option("--password", required=True, help="Пароль")
@click.option("--description", help="Описание сервера")
@click.option(
    "--default", "is_default", is_flag=True, help="Установить как сервер по умолчанию"
)
@click.option(
    "--ttl",
    multiple=True,
    help="Настройки TTL в формате endpoint:ttl (например: /deployments/list:15)",
)
@click.pass_context
def add_server(ctx, name, url, username, password, description, is_default, ttl):
    """Добавляет новый сервер"""
    try:
        svc = ServersService()
        svc.add_server(name, url, username, password, description, is_default)

        console.print(f"[green]Сервер '{name}' успешно добавлен[/green]")

        if is_default:
            console.print(
                f"[blue]Сервер '{name}' установлен как сервер по умолчанию[/blue]"
            )

        # Обрабатываем настройки TTL
        if ttl:
            cache_manager = _get_cache_manager()
            base_rules = cache_manager._get_default_rules()

            console.print(f"\n[bold]Настройка TTL для сервера '{name}':[/bold]")

            for ttl_setting in ttl:
                try:
                    if ":" not in ttl_setting:
                        console.print(
                            f"[red]Ошибка: Неверный формат TTL '{ttl_setting}'. Используйте endpoint:ttl[/red]"
                        )
                        continue

                    endpoint, ttl_value = ttl_setting.split(":", 1)
                    ttl_value = int(ttl_value)

                    # Проверяем, существует ли эндпоинт
                    if endpoint not in base_rules:
                        console.print(
                            f"[red]Ошибка: Эндпоинт '{endpoint}' не найден[/red]"
                        )
                        console.print(
                            f"[yellow]Доступные эндпоинты: {', '.join(base_rules.keys())}[/yellow]"
                        )
                        continue

                    # Устанавливаем TTL
                    cache_manager.set_server_ttl(name, endpoint, ttl_value)
                    console.print(f"[green]  {endpoint}: {ttl_value} сек[/green]")

                except ValueError:
                    console.print(
                        f"[red]Ошибка: Неверное значение TTL '{ttl_value}'. Должно быть числом[/red]"
                    )
                except Exception as e:
                    console.print(
                        f"[red]Ошибка установки TTL для '{endpoint}': {e}[/red]"
                    )

    except ValueError as e:
        console.print(f"[red]Ошибка: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Ошибка добавления сервера: {e}[/red]")
        raise click.Abort()


@cli.command("remove")
@click.argument("name")
@click.option("--confirm", is_flag=True, help="Подтвердить удаление без запроса")
@click.pass_context
def remove_server(ctx, name, confirm):
    """Удаляет сервер"""
    try:
        svc = ServersService()

        if not confirm:
            if not click.confirm(f"Вы уверены, что хотите удалить сервер '{name}'?"):
                console.print("[yellow]Удаление отменено[/yellow]")
                return

        svc.remove_server(name)
        console.print(f"[green]Сервер '{name}' успешно удален[/green]")

    except ValueError as e:
        console.print(f"[red]Ошибка: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Ошибка удаления сервера: {e}[/red]")
        raise click.Abort()


@cli.command("set-default")
@click.argument("name")
@click.pass_context
def set_default_server(ctx, name):
    """Устанавливает сервер по умолчанию"""
    try:
        svc = ServersService()
        svc.set_default_server(name)
        console.print(
            f"[green]Сервер '{name}' установлен как сервер по умолчанию[/green]"
        )

    except ValueError as e:
        console.print(f"[red]Ошибка: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Ошибка установки сервера по умолчанию: {e}[/red]")
        raise click.Abort()


@cli.command("test")
@click.argument("name")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.pass_context
def test_server(ctx, name, no_headers):
    """Тестирует подключение к серверу"""
    try:
        svc = ServersService()
        config = svc.get_config_for_server(name)

        from core.api_client import ImagenariumAPIClient

        api = ImagenariumAPIClient(config, server_name=name)

        # Показываем заголовок консоли
        show_console_header(api, show_header=not no_headers)

        console.print(f"[blue]Тестирование подключения к серверу '{name}'...[/blue]")
        version = api.get_version()
        console.print(
            f"[green]✓ Подключение успешно! Версия «Imagenarium»: {version}[/green]"
        )

    except ValueError as e:
        console.print(f"[red]Ошибка: {e}[/red]")
        raise click.Abort()
    except ImagenariumAPIError as e:
        console.print(f"[red]✗ Ошибка подключения: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]✗ Неожиданная ошибка: {e}[/red]")
        raise click.Abort()


@cli.command("update")
@click.argument("name")
@click.option("--url", help="Новый URL сервера")
@click.option("--username", help="Новое имя пользователя")
@click.option("--password", help="Новый пароль")
@click.option("--description", help="Новое описание")
@click.option(
    "--default", "is_default", is_flag=True, help="Установить как сервер по умолчанию"
)
@click.option(
    "--ttl",
    multiple=True,
    help="Настройки TTL в формате endpoint:ttl (например: /deployments/list:15)",
)
@click.pass_context
def update_server(ctx, name, url, username, password, description, is_default, ttl):
    """Обновляет информацию о сервере"""
    try:
        svc = ServersService()

        # Собираем параметры для обновления
        update_params = {}
        if url:
            update_params["url"] = url
        if username:
            update_params["username"] = username
        if password:
            update_params["password"] = password
        if description is not None:
            update_params["description"] = description
        if is_default:
            update_params["is_default"] = True

        if not update_params:
            console.print("[yellow]Нет параметров для обновления[/yellow]")
            return

        svc.update_server(name, **update_params)
        console.print(f"[green]Сервер '{name}' успешно обновлен[/green]")

        # Обрабатываем настройки TTL
        if ttl:
            cache_manager = _get_cache_manager()
            base_rules = cache_manager._get_default_rules()

            console.print(f"\n[bold]Обновление TTL для сервера '{name}':[/bold]")

            for ttl_setting in ttl:
                try:
                    if ":" not in ttl_setting:
                        console.print(
                            f"[red]Ошибка: Неверный формат TTL '{ttl_setting}'. Используйте endpoint:ttl[/red]"
                        )
                        continue

                    endpoint, ttl_value = ttl_setting.split(":", 1)
                    ttl_value = int(ttl_value)

                    # Проверяем, существует ли эндпоинт
                    if endpoint not in base_rules:
                        console.print(
                            f"[red]Ошибка: Эндпоинт '{endpoint}' не найден[/red]"
                        )
                        console.print(
                            f"[yellow]Доступные эндпоинты: {', '.join(base_rules.keys())}[/yellow]"
                        )
                        continue

                    # Устанавливаем TTL
                    cache_manager.set_server_ttl(name, endpoint, ttl_value)
                    console.print(f"[green]  {endpoint}: {ttl_value} сек[/green]")

                except ValueError:
                    console.print(
                        f"[red]Ошибка: Неверное значение TTL '{ttl_value}'. Должно быть числом[/red]"
                    )
                except Exception as e:
                    console.print(
                        f"[red]Ошибка установки TTL для '{endpoint}': {e}[/red]"
                    )

    except ValueError as e:
        console.print(f"[red]Ошибка: {e}[/red]")
        raise click.Abort()
    except Exception as e:
        console.print(f"[red]Ошибка обновления сервера: {e}[/red]")
        raise click.Abort()


@cli.command("get")
@click.argument("name")
@click.option("--no-cache", is_flag=True, help="Отключить кэширование")
@click.option("--verbose", "-v", is_flag=True, help="Подробный вывод HTTP запросов")
@click.option("--no-headers", is_flag=True, help="Не показывать заголовок консоли")
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["table", "json", "yaml"]),
    default="table",
    help="Формат вывода",
)
@click.pass_context
def get_server(ctx, name, no_cache, verbose, no_headers, output_format):
    """Показывает детальную информацию о сервере и настройки TTL"""
    try:
        # ServersService читает из конфига, не нужен прогресс-бар
        svc = ServersService()
        server = svc.get_server(name)

        if not server:
            console.print(f"[red]Сервер '{name}' не найден[/red]")
            return

        # Показываем заголовок консоли (только для table формата и если не отключен)
        if output_format == "table" and not no_headers:
            header_shown = False
            try:
                from core.api_client import ImagenariumAPIClient

                config = svc.get_config_for_server(name)
                api = ImagenariumAPIClient(config, server_name=name)
                # Пытаемся получить конфиг консоли
                config_data = api.get_console_config()
                console_name = config_data.get("consoleName", server.name)
                console_color = config_data.get("consoleColor", "#666666")

                # Пытаемся получить версию
                try:
                    version = api.get_version(no_cache=True)
                except Exception:
                    version = None

                from utils.formatters import display_console_header

                display_console_header(console_name, console_color, version)
                header_shown = True
            except Exception:
                pass

            # Fallback заголовок если сервер недоступен
            if not header_shown:
                from utils.formatters import display_console_header

                display_console_header(
                    f"Imagenarium: {server.name} (недоступен)", "#666666"
                )

        # Получаем настройки TTL для сервера
        cache_manager = _get_cache_manager()
        server_ttl_settings = cache_manager.get_server_ttl_settings(name)

        if output_format == "table":
            # Выводим основную информацию в стиле других команд get
            console.print(f"[bold white]NAME:[/bold white] {server.name}")
            console.print(f"[bold white]URL:[/bold white] {server.url}")
            console.print(
                f"[bold white]USERNAME:[/bold white] {server.username or 'не указан'}"
            )
            console.print(
                f"[bold white]DESCRIPTION:[/bold white] {server.description or 'не указано'}"
            )
            console.print(
                f"[bold white]DEFAULT:[/bold white] {'✓' if server.is_default else '✗'}"
            )

            # Получаем и выводим версию Imagenarium
            try:
                from core.api_client import ImagenariumAPIClient

                config = svc.get_config_for_server(name)
                api = ImagenariumAPIClient(config, server_name=name)
                version = api.get_version(no_cache=True)
                console.print(f"[bold white]VERSION:[/bold white] {version}")
            except Exception:
                console.print(
                    f"[bold white]VERSION:[/bold white] [red]недоступна[/red]"
                )

            # Настройки TTL
            console.print(f"\n[bold white]TTL:[/bold white]")

            # Получаем пользовательские настройки TTL
            ttl_overrides = {}
            if server_ttl_settings and "ttl_overrides" in server_ttl_settings:
                ttl_overrides = server_ttl_settings["ttl_overrides"]

            # Получаем глобальные настройки
            base_rules = cache_manager._get_default_rules()

            # Показываем все эндпоинты с текущими и дефолтными значениями
            for endpoint, rule in base_rules.items():
                # Определяем текущее значение TTL
                current_ttl = ttl_overrides.get(endpoint, rule["ttl"])
                default_ttl = rule["ttl"]

                # Форматируем текущее значение
                if current_ttl == 0:
                    current_display = f"[red]{current_ttl} сек[/red]"
                elif current_ttl < 60:
                    current_display = f"[yellow]{current_ttl} сек[/yellow]"
                else:
                    current_display = f"[green]{current_ttl} сек[/green]"

                # Форматируем дефолтное значение
                if default_ttl == 0:
                    default_display = f"[red]{default_ttl} сек[/red]"
                elif default_ttl < 60:
                    default_display = f"[yellow]{default_ttl} сек[/yellow]"
                else:
                    default_display = f"[green]{default_ttl} сек[/green]"

                # Показываем значение с указанием дефолтного
                if current_ttl == default_ttl:
                    console.print(f"  {endpoint:<25}: {current_display}")
                else:
                    console.print(
                        f"  {endpoint:<25}: {current_display} (default: {default_display})"
                    )

        elif output_format == "json":
            import json

            # Получаем версию для JSON формата
            version = None
            try:
                from core.api_client import ImagenariumAPIClient

                config = svc.get_config_for_server(name)
                api = ImagenariumAPIClient(config, server_name=name)
                version = api.get_version(no_cache=True)
            except Exception:
                pass

            result = {
                "server": {
                    "name": server.name,
                    "url": server.url,
                    "username": server.username,
                    "description": server.description,
                    "is_default": server.is_default,
                    "version": version,
                },
                "ttl_settings": server_ttl_settings,
                "global_ttl_settings": cache_manager._get_default_rules(),
            }
            console.print(json.dumps(result, indent=2, ensure_ascii=False))

        elif output_format == "yaml":
            import yaml

            # Получаем версию для YAML формата
            version = None
            try:
                from core.api_client import ImagenariumAPIClient

                config = svc.get_config_for_server(name)
                api = ImagenariumAPIClient(config, server_name=name)
                version = api.get_version(no_cache=True)
            except Exception:
                pass

            result = {
                "server": {
                    "name": server.name,
                    "url": server.url,
                    "username": server.username,
                    "description": server.description,
                    "is_default": server.is_default,
                    "version": version,
                },
                "ttl_settings": server_ttl_settings,
                "global_ttl_settings": cache_manager._get_default_rules(),
            }
            console.print(
                yaml.dump(result, default_flow_style=False, allow_unicode=True)
            )

    except Exception as e:
        console.print(f"[red]Ошибка при получении информации о сервере: {e}[/red]")
        raise click.Abort()


@cli.group("cache")
def cache_group():
    """Управление настройками кеширования"""
    pass


@cache_group.command("show")
@click.option("--server", "-s", help="Показать настройки для конкретного сервера")
@click.option(
    "--format",
    "output_format",
    type=click.Choice(["table", "json", "yaml"]),
    default="table",
    help="Формат вывода",
)
@click.pass_context
def show_cache_settings(ctx, server, output_format):
    """Показывает текущие настройки TTL для всех эндпоинтов"""
    try:
        # Создаем CacheManager для получения правил кеширования
        cache_manager = CacheManager()

        # Если сервер не указан, пытаемся получить из контекста
        if not server:
            # Проверяем, есть ли глобальные опции сервера
            if hasattr(ctx, "obj") and ctx.obj and "server" in ctx.obj:
                server = ctx.obj["server"]
            # Или из переменной окружения
            elif "IMG_SERVER" in os.environ:
                server = os.environ["IMG_SERVER"]

        if server:
            # Показываем настройки для конкретного сервера
            server_settings = cache_manager.get_server_ttl_settings(server)
            if not server_settings:
                console.print(
                    f"[yellow]Настройки для сервера '{server}' не найдены[/yellow]"
                )
                console.print(
                    "[yellow]Используются глобальные настройки по умолчанию[/yellow]"
                )
                server_settings = {}

            # Получаем базовые правила
            base_rules = cache_manager._get_default_rules()

            # Создаем объединенные правила
            rules = {}
            for endpoint, rule in base_rules.items():
                rules[endpoint] = rule.copy()
                # Применяем сервер-специфичные настройки
                if "ttl_overrides" in server_settings:
                    if endpoint in server_settings["ttl_overrides"]:
                        rules[endpoint]["ttl"] = server_settings["ttl_overrides"][
                            endpoint
                        ]
        else:
            # Показываем глобальные настройки
            rules = cache_manager._cache_rules

        if output_format == "table":
            # Создаем таблицу
            table = Table(title="Настройки кеширования TTL")
            table.add_column("Эндпоинт", style="cyan", no_wrap=True)
            table.add_column("TTL (сек)", style="green", justify="right")
            table.add_column("Описание", style="white")
            table.add_column("Стратегия", style="blue")

            # Добавляем правила в таблицу
            for endpoint, rule in rules.items():
                ttl_display = f"{rule['ttl']} сек"
                if rule["ttl"] == 0:
                    ttl_display = "[red]0 сек (всегда обновляется)[/red]"
                elif rule["ttl"] < 60:
                    ttl_display = (
                        f"[yellow]{rule['ttl']} сек (быстрое обновление)[/yellow]"
                    )
                else:
                    ttl_display = f"[green]{rule['ttl']} сек[/green]"

                table.add_row(
                    endpoint, ttl_display, rule["description"], rule["strategy"]
                )

            console.print(table)

            # Показываем информацию о сервере, если указан
            if server:
                console.print(f"\n[bold]Настройки для сервера: {server}[/bold]")
                console.print(
                    "[yellow]Примечание: TTL настройки глобальные для всех серверов[/yellow]"
                )
            else:
                console.print(f"\n[bold]Всего правил кеширования: {len(rules)}[/bold]")
                console.print(
                    "[yellow]Примечание: TTL настройки глобальные для всех серверов[/yellow]"
                )

        elif output_format == "json":
            import json

            result = {
                "cache_settings": rules,
                "note": "TTL настройки глобальные для всех серверов",
            }
            if server:
                result["server"] = server
            console.print(json.dumps(result, indent=2, ensure_ascii=False))

        elif output_format == "yaml":
            import yaml

            result = {
                "cache_settings": rules,
                "note": "TTL настройки глобальные для всех серверов",
            }
            if server:
                result["server"] = server
            console.print(
                yaml.dump(result, default_flow_style=False, allow_unicode=True)
            )

    except Exception as e:
        console.print(f"[red]Ошибка при получении настроек кеширования: {e}[/red]")
        raise click.Abort()


@cache_group.command("set")
@click.argument("endpoint")
@click.argument("ttl", type=int)
@click.option("--server", "-s", help="Установить настройку для конкретного сервера")
@click.pass_context
def set_cache_ttl(ctx, endpoint, ttl, server):
    """Устанавливает TTL для конкретного эндпоинта"""
    try:
        if ttl < 0:
            console.print("[red]Ошибка: TTL не может быть отрицательным[/red]")
            raise click.Abort()

        # Если сервер не указан, пытаемся получить из контекста
        if not server:
            # Проверяем, есть ли глобальные опции сервера
            if hasattr(ctx, "obj") and ctx.obj and "server" in ctx.obj:
                server = ctx.obj["server"]
            # Или из переменной окружения
            elif "IMG_SERVER" in os.environ:
                server = os.environ["IMG_SERVER"]

        # Создаем CacheManager
        cache_manager = CacheManager()

        # Проверяем, существует ли эндпоинт
        base_rules = cache_manager._get_default_rules()
        if endpoint not in base_rules:
            console.print(f"[red]Ошибка: Эндпоинт '{endpoint}' не найден[/red]")
            console.print(
                f"[yellow]Доступные эндпоинты: {', '.join(base_rules.keys())}[/yellow]"
            )
            raise click.Abort()

        if server:
            # Устанавливаем TTL для конкретного сервера
            try:
                cache_manager.set_server_ttl(server, endpoint, ttl)
                console.print(
                    f"[green]TTL для '{endpoint}' на сервере '{server}' установлен в {ttl} сек[/green]"
                )
            except Exception as e:
                console.print(f"[red]Ошибка установки TTL для сервера: {e}[/red]")
                raise click.Abort()
        else:
            # Устанавливаем глобальный TTL
            old_ttl = cache_manager._cache_rules[endpoint]["ttl"]
            cache_manager._cache_rules[endpoint]["ttl"] = ttl
            cache_manager._save_user_config()
            console.print(
                f"[green]Глобальный TTL для '{endpoint}' изменен с {old_ttl} сек на {ttl} сек[/green]"
            )

        if ttl == 0:
            console.print(
                "[yellow]Эндпоинт будет всегда обновляться (кеширование отключено)[/yellow]"
            )
        elif ttl < 60:
            console.print(
                "[yellow]Эндпоинт будет обновляться быстро (частые запросы)[/yellow]"
            )
        else:
            console.print("[green]Эндпоинт будет кешироваться эффективно[/green]")

    except Exception as e:
        console.print(f"[red]Ошибка при установке TTL: {e}[/red]")
        raise click.Abort()


@cache_group.command("reset")
@click.option("--server", "-s", help="Сбросить настройки для конкретного сервера")
@click.confirmation_option(
    prompt="Вы уверены, что хотите сбросить настройки кеширования?"
)
@click.pass_context
def reset_cache_settings(ctx, server):
    """Сбрасывает настройки кеширования к значениям по умолчанию"""
    try:
        # Если сервер не указан, пытаемся получить из контекста
        if not server:
            # Проверяем, есть ли глобальные опции сервера
            if hasattr(ctx, "obj") and ctx.obj and "server" in ctx.obj:
                server = ctx.obj["server"]
            # Или из переменной окружения
            elif "IMG_SERVER" in os.environ:
                server = os.environ["IMG_SERVER"]

        if server:
            console.print(
                f"[yellow]Внимание: Сброс настроек для сервера '{server}' пока не поддерживается[/yellow]"
            )
            console.print(
                "[yellow]Настройки будут сброшены глобально для всех серверов[/yellow]"
            )

        # Создаем CacheManager
        cache_manager = CacheManager()

        # Сбрасываем кеш
        cache_manager.clear_cache()

        console.print(
            "[green]Настройки кеширования сброшены к значениям по умолчанию[/green]"
        )
        console.print("[yellow]Все кешированные данные очищены[/yellow]")

    except Exception as e:
        console.print(f"[red]Ошибка при сбросе настроек: {e}[/red]")
        raise click.Abort()



"""
Сервисы данных для команд и комплитера

Базовые сервисы предоставляют единый интерфейс:
 - list(filters)
 - get(key_or_name)
 - get_available_columns()
 - get_default_columns()
 - get_completions(column, current_filters, operator, prefix)
"""

from .base import BaseListService  # noqa: F401
from .components_service import ComponentsService  # noqa: F401
from .nodes_service import NodesService  # noqa: F401
from .registries_service import RegistriesService  # noqa: F401
from .repositories_service import RepositoriesService  # noqa: F401
from .servers_service import ServersService  # noqa: F401
from .stacks_service import StacksService  # noqa: F401
